from unittest.mock import patch

from django_webtest import WebTest

from django.conf import settings
from django.contrib import admin
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.test import TestCase, RequestFactory, Client

from allianceauth.authentication.models import CharacterOwnership, State
from allianceauth.eveonline.models import (
    EveCharacter, EveCorporationInfo, EveAllianceInfo
)
from allianceauth.tests.auth_utils import AuthUtils

from . import get_admin_change_view_url
from ..admin import HasLeaderFilter, GroupAdmin, Group
from ..models import ReservedGroupName


if 'allianceauth.eveonline.autogroups' in settings.INSTALLED_APPS:
    _has_auto_groups = True
    from allianceauth.eveonline.autogroups.models import AutogroupsConfig
    from ..admin import IsAutoGroupFilter
else:
    _has_auto_groups = False


MODULE_PATH = 'allianceauth.groupmanagement.admin'


class MockRequest:

    def __init__(self, user=None):
        self.user = user


class TestGroupAdmin(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        # group 1 - has leader
        cls.group_1 = Group.objects.create(name='Group 1')
        cls.group_1.authgroup.description = 'Default Group'
        cls.group_1.authgroup.internal = False
        cls.group_1.authgroup.hidden = False
        cls.group_1.authgroup.save()

        # group 2 - no leader
        cls.group_2 = Group.objects.create(name='Group 2')
        cls.group_2.authgroup.description = 'Internal Group'
        cls.group_2.authgroup.internal = True
        cls.group_2.authgroup.group_leader_groups.add(cls.group_1)
        cls.group_2.authgroup.save()

        # group 3 - has leader
        cls.group_3 = Group.objects.create(name='Group 3')
        cls.group_3.authgroup.description = 'Hidden Group'
        cls.group_3.authgroup.internal = False
        cls.group_3.authgroup.hidden = True
        cls.group_3.authgroup.save()

        # group 4 - no leader
        cls.group_4 = Group.objects.create(name='Group 4')
        cls.group_4.authgroup.description = 'Open Group'
        cls.group_4.authgroup.internal = False
        cls.group_4.authgroup.hidden = False
        cls.group_4.authgroup.open = True
        cls.group_4.authgroup.save()

        # group 5 - no leader
        cls.group_5 = Group.objects.create(name='Group 5')
        cls.group_5.authgroup.description = 'Public Group'
        cls.group_5.authgroup.internal = False
        cls.group_5.authgroup.hidden = False
        cls.group_5.authgroup.public = True
        cls.group_5.authgroup.save()

        # group 6 - no leader
        cls.group_6 = Group.objects.create(name='Group 6')
        cls.group_6.authgroup.description = 'Mixed Group'
        cls.group_6.authgroup.internal = False
        cls.group_6.authgroup.hidden = True
        cls.group_6.authgroup.open = True
        cls.group_6.authgroup.public = True
        cls.group_6.authgroup.save()

        # user 1 - corp and alliance, normal user
        cls.character_1 = EveCharacter.objects.create(
            character_id=1001,
            character_name='Bruce Wayne',
            corporation_id=2001,
            corporation_name='Wayne Technologies',
            corporation_ticker='WT',
            alliance_id=3001,
            alliance_name='Wayne Enterprises',
            alliance_ticker='WE',
        )
        cls.character_1a = EveCharacter.objects.create(
            character_id=1002,
            character_name='Batman',
            corporation_id=2001,
            corporation_name='Wayne Technologies',
            corporation_ticker='WT',
            alliance_id=3001,
            alliance_name='Wayne Enterprises',
            alliance_ticker='WE',
        )
        alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Wayne Enterprises',
            alliance_ticker='WE',
            executor_corp_id=2001
        )
        EveCorporationInfo.objects.create(
            corporation_id=2001,
            corporation_name='Wayne Technologies',
            corporation_ticker='WT',
            member_count=42,
            alliance=alliance
        )
        cls.user_1 = User.objects.create_user(
            cls.character_1.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=cls.character_1,
            owner_hash='x1' + cls.character_1.character_name,
            user=cls.user_1
        )
        CharacterOwnership.objects.create(
            character=cls.character_1a,
            owner_hash='x1' + cls.character_1a.character_name,
            user=cls.user_1
        )
        cls.user_1.profile.main_character = cls.character_1
        cls.user_1.profile.save()
        cls.user_1.groups.add(cls.group_1)
        cls.group_1.authgroup.group_leaders.add(cls.user_1)

        # user 2 - corp only, staff
        cls.character_2 = EveCharacter.objects.create(
            character_id=1003,
            character_name='Clark Kent',
            corporation_id=2002,
            corporation_name='Daily Planet',
            corporation_ticker='DP',
            alliance_id=None
        )
        EveCorporationInfo.objects.create(
            corporation_id=2002,
            corporation_name='Daily Plane',
            corporation_ticker='DP',
            member_count=99,
            alliance=None
        )
        cls.user_2 = User.objects.create_user(
            cls.character_2.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=cls.character_2,
            owner_hash='x1' + cls.character_2.character_name,
            user=cls.user_2
        )
        cls.user_2.profile.main_character = cls.character_2
        cls.user_2.profile.save()
        cls.user_2.groups.add(cls.group_2)
        cls.user_2.is_staff = True
        cls.user_2.save()

        # user 3 - no main, no group, superuser
        cls.character_3 = EveCharacter.objects.create(
            character_id=1101,
            character_name='Lex Luthor',
            corporation_id=2101,
            corporation_name='Lex Corp',
            corporation_ticker='LC',
            alliance_id=None
        )
        EveCorporationInfo.objects.create(
            corporation_id=2101,
            corporation_name='Lex Corp',
            corporation_ticker='LC',
            member_count=666,
            alliance=None
        )
        EveAllianceInfo.objects.create(
            alliance_id=3101,
            alliance_name='Lex World Domination',
            alliance_ticker='LWD',
            executor_corp_id=2101
        )
        cls.user_3 = User.objects.create_user(
            cls.character_3.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=cls.character_3,
            owner_hash='x1' + cls.character_3.character_name,
            user=cls.user_3
        )
        cls.user_3.is_superuser = True
        cls.user_3.save()
        cls.user_3.groups.add(cls.group_3)
        cls.group_3.authgroup.group_leaders.add(cls.user_3)

    def setUp(self):
        self.factory = RequestFactory()
        self.modeladmin = GroupAdmin(
            model=Group, admin_site=AdminSite()
        )

    def _create_autogroups(self):
        """create autogroups for corps and alliances"""
        if _has_auto_groups:
            autogroups_config = AutogroupsConfig(
                corp_groups=True,
                alliance_groups=True
            )
            autogroups_config.save()
            for state in State.objects.all():
                autogroups_config.states.add(state)
            autogroups_config.update_corp_group_membership(self.user_1)

    # column rendering

    def test_description(self):
        expected = 'Default Group'
        result = self.modeladmin._description(self.group_1)
        self.assertEqual(result, expected)

    def test_member_count(self):
        expected = 1
        obj = self.modeladmin.get_queryset(MockRequest(user=self.user_1))\
            .get(pk=self.group_1.pk)
        result = self.modeladmin._member_count(obj)
        self.assertEqual(result, expected)

    def test_has_leader_user(self):
        result = self.modeladmin.has_leader(self.group_1)
        self.assertTrue(result)

    def test_has_leader_group(self):
        result = self.modeladmin.has_leader(self.group_2)
        self.assertTrue(result)

    def test_properties_1(self):
        expected = ['Default']
        result = self.modeladmin._properties(self.group_1)
        self.assertListEqual(result, expected)

    def test_properties_2(self):
        expected = ['Internal']
        result = self.modeladmin._properties(self.group_2)
        self.assertListEqual(result, expected)

    def test_properties_3(self):
        expected = ['Hidden']
        result = self.modeladmin._properties(self.group_3)
        self.assertListEqual(result, expected)

    def test_properties_4(self):
        expected = ['Open']
        result = self.modeladmin._properties(self.group_4)
        self.assertListEqual(result, expected)

    def test_properties_5(self):
        expected = ['Public']
        result = self.modeladmin._properties(self.group_5)
        self.assertListEqual(result, expected)

    def test_properties_6(self):
        expected = ['Hidden', 'Open', 'Public']
        result = self.modeladmin._properties(self.group_6)
        self.assertListEqual(result, expected)

    if _has_auto_groups:
        @patch(MODULE_PATH + '._has_auto_groups', True)
        def test_properties_7(self):
            self._create_autogroups()
            expected = ['Auto Group']
            my_group = Group.objects\
                .filter(managedcorpgroup__isnull=False)\
                .first()
            result = self.modeladmin._properties(my_group)
            self.assertListEqual(result, expected)

    # actions

    # filters

    if _has_auto_groups:
        @patch(MODULE_PATH + '._has_auto_groups', True)
        def test_filter_is_auto_group(self):

            class GroupAdminTest(admin.ModelAdmin):
                list_filter = (IsAutoGroupFilter,)

            self._create_autogroups()
            my_modeladmin = GroupAdminTest(Group, AdminSite())

            # Make sure the lookups are correct
            request = self.factory.get('/')
            request.user = self.user_1
            changelist = my_modeladmin.get_changelist_instance(request)
            filters = changelist.get_filters(request)
            filterspec = filters[0][0]
            expected = [
                ('yes', 'Yes'),
                ('no', 'No'),
            ]
            self.assertEqual(filterspec.lookup_choices, expected)

            # Make sure the correct queryset is returned - no
            request = self.factory.get(
                '/', {'is_auto_group__exact': 'no'}
            )
            request.user = self.user_1
            changelist = my_modeladmin.get_changelist_instance(request)
            queryset = changelist.get_queryset(request)
            expected = [
                self.group_1,
                self.group_2,
                self.group_3,
                self.group_4,
                self.group_5,
                self.group_6
            ]
            self.assertSetEqual(set(queryset), set(expected))

            # Make sure the correct queryset is returned - yes
            request = self.factory.get(
                '/', {'is_auto_group__exact': 'yes'}
            )
            request.user = self.user_1
            changelist = my_modeladmin.get_changelist_instance(request)
            queryset = changelist.get_queryset(request)
            expected = Group.objects.exclude(
                managedalliancegroup__isnull=True,
                managedcorpgroup__isnull=True
            )
            self.assertSetEqual(set(queryset), set(expected))

    def test_filter_has_leader(self):

        class GroupAdminTest(admin.ModelAdmin):
            list_filter = (HasLeaderFilter,)

        self._create_autogroups()
        my_modeladmin = GroupAdminTest(Group, AdminSite())

        # Make sure the lookups are correct
        request = self.factory.get('/')
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        filters = changelist.get_filters(request)
        filterspec = filters[0][0]
        expected = [
            ('yes', 'Yes'),
            ('no', 'No'),
        ]
        self.assertEqual(filterspec.lookup_choices, expected)

        # Make sure the correct queryset is returned - no
        request = self.factory.get(
            '/', {'has_leader__exact': 'no'}
        )
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        queryset = changelist.get_queryset(request)
        expected = Group.objects.exclude(pk__in=[
            self.group_1.pk, self.group_3.pk
        ])
        self.assertSetEqual(set(queryset), set(expected))

        # Make sure the correct queryset is returned - yes
        request = self.factory.get(
            '/', {'has_leader__exact': 'yes'}
        )
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        queryset = changelist.get_queryset(request)
        expected = [
            self.group_1,
            self.group_3
        ]
        self.assertSetEqual(set(queryset), set(expected))

    def test_change_view_loads_normally(self):
        User.objects.create_superuser(
            username='superuser', password='secret', email='admin@example.com'
        )
        c = Client()
        c.login(username='superuser', password='secret')
        response = c.get(get_admin_change_view_url(self.group_1))
        self.assertEqual(response.status_code, 200)

    def test_should_create_new_group(self):
        # given
        user = User.objects.create_superuser("bruce")
        self.client.force_login(user)
        # when
        response = self.client.post(
            "/admin/groupmanagement/group/add/",
            data={
                "name": "new group",
                "authgroup-TOTAL_FORMS": 1,
                "authgroup-INITIAL_FORMS": 0,
                "authgroup-MIN_NUM_FORMS": 0,
                "authgroup-MAX_NUM_FORMS": 1,
            }
        )
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/admin/groupmanagement/group/")
        self.assertTrue(Group.objects.filter(name="new group").exists())

    def test_should_not_allow_creating_new_group_with_reserved_name(self):
        # given
        ReservedGroupName.objects.create(
            name="new group", reason="dummy", created_by="bruce"
        )
        user = User.objects.create_superuser("bruce")
        self.client.force_login(user)
        # when
        response = self.client.post(
            "/admin/groupmanagement/group/add/",
            data={
                "name": "New group",
                "authgroup-TOTAL_FORMS": 1,
                "authgroup-INITIAL_FORMS": 0,
                "authgroup-MIN_NUM_FORMS": 0,
                "authgroup-MAX_NUM_FORMS": 1,
            }
        )
        # then
        self.assertContains(
            response, "This name has been reserved and can not be used for groups"
        )
        self.assertFalse(Group.objects.filter(name="new group").exists())

    def test_should_not_allow_changing_name_of_existing_group_to_reserved_name(self):
        # given
        ReservedGroupName.objects.create(
            name="new group", reason="dummy", created_by="bruce"
        )
        group = Group.objects.create(name="dummy")
        user = User.objects.create_superuser("bruce")
        self.client.force_login(user)
        # when
        response = self.client.post(
            f"/admin/groupmanagement/group/{group.pk}/change/",
            data={
                "name": "new group",
                "authgroup-TOTAL_FORMS": 1,
                "authgroup-INITIAL_FORMS": 0,
                "authgroup-MIN_NUM_FORMS": 0,
                "authgroup-MAX_NUM_FORMS": 1,
            }
        )
        # then
        self.assertContains(
            response, "This name has been reserved and can not be used for groups"
        )
        self.assertFalse(Group.objects.filter(name="new group").exists())


class TestGroupAdminChangeFormSuperuserExclusiveEdits(WebTest):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.super_admin = User.objects.create_superuser("super_admin")
        cls.staff_admin = User.objects.create_user("staff_admin")
        cls.staff_admin.is_staff = True
        cls.staff_admin.save()
        cls.staff_admin = AuthUtils.add_permissions_to_user_by_name(
            [
                "auth.add_group",
                "auth.change_group",
                "auth.view_group",
                "groupmanagement.add_group",
                "groupmanagement.change_group",
                "groupmanagement.view_group",
            ],
            cls.staff_admin
        )
        cls.superuser_exclusive_fields = ["permissions", "authgroup-0-restricted"]

    def test_should_show_all_fields_to_superuser_for_add(self):
        # given
        self.app.set_user(self.super_admin)
        page = self.app.get("/admin/groupmanagement/group/add/")
        # when
        form = page.forms["group_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertIn(field, form.fields)

    def test_should_not_show_all_fields_to_staff_admins_for_add(self):
        # given
        self.app.set_user(self.staff_admin)
        page = self.app.get("/admin/groupmanagement/group/add/")
        # when
        form = page.forms["group_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertNotIn(field, form.fields)

    def test_should_show_all_fields_to_superuser_for_change(self):
        # given
        self.app.set_user(self.super_admin)
        group = Group.objects.create(name="Dummy group")
        page = self.app.get(f"/admin/groupmanagement/group/{group.pk}/change/")
        # when
        form = page.forms["group_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertIn(field, form.fields)

    def test_should_not_show_all_fields_to_staff_admin_for_change(self):
        # given
        self.app.set_user(self.staff_admin)
        group = Group.objects.create(name="Dummy group")
        page = self.app.get(f"/admin/groupmanagement/group/{group.pk}/change/")
        # when
        form = page.forms["group_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertNotIn(field, form.fields)


class TestReservedGroupNameAdmin(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user = User.objects.create_superuser("bruce")

    def test_should_create_new_entry(self):
        # given
        self.client.force_login(self.user)
        # when
        response = self.client.post(
            "/admin/groupmanagement/reservedgroupname/add/",
            data={"name": "Test", "reason": "dummy"}
        )
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/admin/groupmanagement/reservedgroupname/")
        obj = ReservedGroupName.objects.get(name="test")
        self.assertEqual(obj.name, "test")
        self.assertEqual(obj.created_by, self.user.username)
        self.assertTrue(obj.created_at)

    def test_should_not_allow_names_of_existing_groups(self):
        # given
        Group.objects.create(name="Already taken")
        self.client.force_login(self.user)
        # when
        response = self.client.post(
            "/admin/groupmanagement/reservedgroupname/add/",
            data={"name": "already taken", "reason": "dummy"}
        )
        # then
        self.assertContains(response, "There already exists a group with that name")
        self.assertFalse(ReservedGroupName.objects.filter(name="already taken").exists())



from django.contrib import admin

from .models import MumbleUser
from ...admin import ServicesUserAdmin


@admin.register(MumbleUser)
class MumbleUserAdmin(ServicesUserAdmin):
    list_display = ServicesUserAdmin.list_display + (
        'username',
        'groups',
    )
    search_fields = ServicesUserAdmin.search_fields + (
        'username',
        'groups'
    )

    fields = ('user', 'username', 'groups')  # pwhash is hidden from admin panel



import datetime as dt
from typing import Optional, List

from redis import Redis
from pytz import utc

from django.core.cache import cache


class EventSeries:
    """API for recording and analysing a series of events."""

    _ROOT_KEY = "ALLIANCEAUTH_EVENT_SERIES"

    def __init__(self, key_id: str, redis: Redis = None) -> None:
        self._redis = cache.get_master_client() if not redis else redis
        if not isinstance(self._redis, Redis):
            raise TypeError(
                "This class requires a Redis client, but none was provided "
                "and the default Django cache backend is not Redis either."
            )
        self._key_id = str(key_id)
        self.clear()

    @property
    def _key_counter(self):
        return f"{self._ROOT_KEY}_{self._key_id}_COUNTER"

    @property
    def _key_sorted_set(self):
        return f"{self._ROOT_KEY}_{self._key_id}_SORTED_SET"

    def add(self, event_time: dt.datetime = None) -> None:
        """Add event.

        Args:
        - event_time: timestamp of event. Will use current time if not specified.
        """
        if not event_time:
            event_time = dt.datetime.utcnow()
        id = self._redis.incr(self._key_counter)
        self._redis.zadd(self._key_sorted_set, {id: event_time.timestamp()})

    def all(self) -> List[dt.datetime]:
        """List of all known events."""
        return [
            event[1]
            for event in self._redis.zrangebyscore(
                self._key_sorted_set,
                "-inf",
                "+inf",
                withscores=True,
                score_cast_func=self._cast_scores_to_dt,
            )
        ]

    def clear(self) -> None:
        """Clear all events."""
        self._redis.delete(self._key_sorted_set)
        self._redis.delete(self._key_counter)

    def count(self, earliest: dt.datetime = None, latest: dt.datetime = None) -> int:
        """Count of events, can be restricted to given timeframe.

        Args:
        - earliest: Date of first events to count(inclusive), or -infinite if not specified
        - latest: Date of last events to count(inclusive), or +infinite if not specified
        """
        min = "-inf" if not earliest else earliest.timestamp()
        max = "+inf" if not latest else latest.timestamp()
        return self._redis.zcount(self._key_sorted_set, min=min, max=max)

    def first_event(self, earliest: dt.datetime = None) -> Optional[dt.datetime]:
        """Date/Time of first event. Returns `None` if series has no events.

        Args:
        - earliest: Date of first events to count(inclusive), or any if not specified
        """
        min = "-inf" if not earliest else earliest.timestamp()
        event = self._redis.zrangebyscore(
            self._key_sorted_set,
            min,
            "+inf",
            withscores=True,
            start=0,
            num=1,
            score_cast_func=self._cast_scores_to_dt,
        )
        if not event:
            return None
        return event[0][1]

    @staticmethod
    def _cast_scores_to_dt(score) -> dt.datetime:
        return dt.datetime.fromtimestamp(float(score), tz=utc)



# Generated by Django 1.10.5 on 2017-02-06 08:58

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PermissionsTool',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'managed': False,
                'permissions': (('audit_permissions', 'Can audit permissions'),),
            },
        ),
    ]



from unittest.mock import Mock, patch
from django.test import TestCase

from .. import app_settings

MODULE_PATH = 'allianceauth.authentication'


class TestSetAppSetting(TestCase):

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_if_not_set(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = Mock(spec=None)
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            False,
        )
        self.assertEqual(result, False)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_if_not_set_for_none(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = Mock(spec=None)
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            None,
            required_type=int
        )
        self.assertEqual(result, None)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_true_stays_true(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = True
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            False,
        )
        self.assertEqual(result, True)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_false_stays_false(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = False
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            False
        )
        self.assertEqual(result, False)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_for_invalid_type_bool(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = 'invalid type'
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            False
        )
        self.assertEqual(result, False)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_for_invalid_type_int(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = 'invalid type'
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            50
        )
        self.assertEqual(result, 50)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_if_below_minimum_1(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = -5
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            default_value=50
        )
        self.assertEqual(result, 50)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_if_below_minimum_2(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = -50
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            default_value=50,
            min_value=-10
        )
        self.assertEqual(result, 50)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_for_invalid_type_int(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = 1000
        result = app_settings._clean_setting(
            'TEST_SETTING_DUMMY',
            default_value=50,
            max_value=100
        )
        self.assertEqual(result, 50)

    @patch(MODULE_PATH + '.app_settings.settings')
    def test_default_is_none_needs_required_type(self, mock_settings):
        mock_settings.TEST_SETTING_DUMMY = 'invalid type'
        with self.assertRaises(ValueError):
            result = app_settings._clean_setting(
                'TEST_SETTING_DUMMY',
                default_value=None
            )



import requests
import logging
from django.conf import settings
from django.apps import apps
from celery import shared_task
from allianceauth import __version__
from .models import AnalyticsTokens, AnalyticsIdentifier
from .utils import (
    install_stat_addons,
    install_stat_tokens,
    install_stat_users)

logger = logging.getLogger(__name__)

BASE_URL = "https://www.google-analytics.com/"

DEBUG_URL = f"{BASE_URL}debug/collect"
COLLECTION_URL = f"{BASE_URL}collect"

if getattr(settings, "ANALYTICS_ENABLE_DEBUG", False) and settings.DEBUG:
    # Force sending of analytics data during in a debug/test environemt
    # Usefull for developers working on this feature.
    logger.warning(
        "You have 'ANALYTICS_ENABLE_DEBUG' Enabled! "
        "This debug instance will send analytics data!")
    DEBUG_URL = COLLECTION_URL

ANALYTICS_URL = COLLECTION_URL

if settings.DEBUG is True:
    ANALYTICS_URL = DEBUG_URL


def analytics_event(category: str,
                    action: str,
                    label: str,
                    value: int = 0,
                    event_type: str = 'Celery'):
    """
    Send a Google Analytics Event for each token stored
    Includes check for if its enabled/disabled

    Args:
        `category` (str): Celery Namespace
        `action` (str): Task Name
        `label` (str): Optional, Task Success/Exception
        `value` (int): Optional, If bulk, Query size, can be a binary True/False
        `event_type` (str): Optional, Celery or Stats only, Default to Celery
    """
    analyticstokens = AnalyticsTokens.objects.all()
    client_id = AnalyticsIdentifier.objects.get(id=1).identifier.hex
    for token in analyticstokens:
        if event_type == 'Celery':
            allowed = token.send_celery_tasks
        elif event_type == 'Stats':
            allowed = token.send_stats
        else:
            allowed = False

        if allowed is True:
            tracking_id = token.token
            send_ga_tracking_celery_event.s(
                tracking_id=tracking_id,
                client_id=client_id,
                category=category,
                action=action,
                label=label,
                value=value).apply_async(priority=9)


@shared_task()
def analytics_daily_stats():
    """Celery Task: Do not call directly

    Gathers a series of daily statistics and sends analytics events containing them
    """
    users = install_stat_users()
    tokens = install_stat_tokens()
    addons = install_stat_addons()
    logger.debug("Running Daily Analytics Upload")

    analytics_event(category='allianceauth.analytics',
                    action='send_install_stats',
                    label='existence',
                    value=1,
                    event_type='Stats')
    analytics_event(category='allianceauth.analytics',
                    action='send_install_stats',
                    label='users',
                    value=users,
                    event_type='Stats')
    analytics_event(category='allianceauth.analytics',
                    action='send_install_stats',
                    label='tokens',
                    value=tokens,
                    event_type='Stats')
    analytics_event(category='allianceauth.analytics',
                    action='send_install_stats',
                    label='addons',
                    value=addons,
                    event_type='Stats')

    for appconfig in apps.get_app_configs():
        analytics_event(category='allianceauth.analytics',
                        action='send_extension_stats',
                        label=appconfig.label,
                        value=1,
                        event_type='Stats')


@shared_task()
def send_ga_tracking_web_view(
                            tracking_id: str,
                            client_id: str,
                            page: str,
                            title: str,
                            locale: str,
                            useragent: str) -> requests.Response:

    """Celery Task: Do not call directly

    Sends Page View events to GA, Called only via analytics.middleware

    Parameters
    ----------
    `tracking_id` (str): Unique Server Identifier
    `client_id` (str): GA Token
    `page` (str): Page Path
    `title` (str): Page Title
    `locale` (str): Browser Language
    `useragent` (str): Browser UserAgent

    Returns
    -------
    requests.Reponse Object
    """
    headers = {"User-Agent": useragent}

    payload = {
            'v': '1',
            'tid': tracking_id,
            'cid': client_id,
            't': 'pageview',
            'dp': page,
            'dt': title,
            'ul': locale,
            'ua': useragent,
            'aip': 1,
            'an': "allianceauth",
            'av': __version__
            }

    response = requests.post(
                        ANALYTICS_URL, data=payload,
                        timeout=5, headers=headers)
    logger.debug(f"Analytics Page View HTTP{response.status_code}")
    return response


@shared_task()
def send_ga_tracking_celery_event(
                                tracking_id: str,
                                client_id: str,
                                category: str,
                                action: str,
                                label: str,
                                value: int) -> requests.Response:
    """Celery Task: Do not call directly

    Sends Page View events to GA, Called only via analytics.middleware

    Parameters
    ----------
    `tracking_id` (str): Unique Server Identifier
    `client_id` (str): GA Token
    `category` (str): Celery Namespace
    `action` (str): Task Name
    `label` (str): Optional, Task Success/Exception
    `value` (int): Optional, If bulk, Query size, can be a binary True/False

    Returns
    -------
    requests.Reponse Object
    """

    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36"}

    payload = {
            'v': '1',
            'tid': tracking_id,
            'cid': client_id,
            't': 'event',
            'ec': category,
            'ea': action,
            'el': label,
            'ev': value,
            'aip': 1,
            'an': "allianceauth",
            'av': __version__
            }

    response = requests.post(
                        ANALYTICS_URL, data=payload,
                        timeout=5, headers=headers)
    logger.debug(f"Analytics Celery/Stats Event HTTP{response.status_code}")
    return response



from django.urls import reverse


def get_admin_change_view_url(obj: object) -> str:
    return reverse(
        'admin:{}_{}_change'.format(
            obj._meta.app_label,
            type(obj).__name__.lower()
        ),
        args=(obj.pk,)
    )



# Generated by Django 3.1.6 on 2021-03-23 13:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0001_squashed_0011_auto_20201011_1009'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='last_connect',
            field=models.DateTimeField(blank=True, editable=False, help_text='Timestamp of the users Last Connection to Mumble', max_length=254, null=True, verbose_name='Last Connection'),
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='last_disconnect',
            field=models.DateTimeField(blank=True, editable=False, help_text='Timestamp of the users Last Disconnection to Mumble', max_length=254, null=True, verbose_name='Last Disconnection'),
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='release',
            field=models.TextField(blank=True, editable=False, help_text='The Mumble Release the user last authenticated with', max_length=254, null=True, verbose_name='Mumble Release'),
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='version',
            field=models.IntegerField(blank=True, editable=False, help_text='Client version. Major version in upper 16 bits, followed by 8 bits of minor version and 8 bits of patchlevel. Version 1.2.3 = 0x010203.', null=True, verbose_name='Mumble Version'),
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-08 13:54

from django.conf import settings
from django.db import migrations, models
import django.db.migrations.operations.special
import django.db.models.deletion

class Migration(migrations.Migration):

    replaces = [('mumble', '0001_initial'), ('mumble', '0002_auto_20161212_0100'), ('mumble', '0003_mumbleuser_user'), ('mumble', '0004_auto_20161214_1024'), ('mumble', '0005_mumbleuser_hashfn'), ('mumble', '0006_service_permissions'), ('mumble', '0007_not_null_user'), ('mumble', '0008_mumbleuser_display_name'), ('mumble', '0009_set_mumble_dissplay_names'), ('mumble', '0010_mumbleuser_certhash'), ('mumble', '0011_auto_20201011_1009')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MumbleUser',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='mumble', serialize=False, to=settings.AUTH_USER_MODEL)),
                ('username', models.CharField(max_length=254, unique=True)),
                ('pwhash', models.CharField(max_length=40)),
                ('groups', models.TextField(blank=True, null=True)),
            ],
            options={
                'db_table': 'services_mumbleuser',
            },
        ),
        migrations.AlterModelTable(
            name='mumbleuser',
            table=None,
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='hashfn',
            field=models.CharField(default='sha1', max_length=20),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='pwhash',
            field=models.CharField(max_length=80),
        ),
        migrations.AlterModelOptions(
            name='mumbleuser',
            options={'permissions': (('access_mumble', 'Can access the Mumble service'),)},
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='display_name',
            field=models.CharField(max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='display_name',
            field=models.CharField(max_length=254, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='certhash',
            field=models.CharField(blank=True, editable=False, help_text='Hash of Mumble client certificate as presented when user authenticates', max_length=254, null=True, verbose_name='Certificate Hash'),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='pwhash',
            field=models.CharField(max_length=90),
        ),
    ]



"""Templatetags for notifications

These template tags are required to enable the notifications refresh functionality
in the browser.
"""

import logging

from django import template
from django.conf import settings
from django.contrib.auth.models import User

from allianceauth.notifications.models import Notification


logger = logging.getLogger(__name__)
register = template.Library()


@register.filter
def user_unread_notification_count(user: object) -> int:
    """returns the number of unread notifications for user

    Will return -1 on error
    """
    if not isinstance(user, User):
        unread_count = -1
    else:
        unread_count = Notification.objects.user_unread_count(user.pk)

    return unread_count


@register.simple_tag
def notifications_refresh_time() -> int:
    refresh_time = getattr(settings, 'NOTIFICATIONS_REFRESH_TIME', Notification.NOTIFICATIONS_REFRESH_TIME_DEFAULT)
    if (not isinstance(refresh_time, int) or refresh_time < 0):
        logger.warning('NOTIFICATIONS_REFRESH_TIME setting is invalid. Using default.')
        refresh_time = Notification.NOTIFICATIONS_REFRESH_TIME_DEFAULT

    return refresh_time



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _

from allianceauth.services.views import superuser_test

from . import __title__
from .models import DiscordUser
from .utils import LoggerAddTag


logger = LoggerAddTag(logging.getLogger(__name__), __title__)

ACCESS_PERM = 'discord.access_discord'


@login_required
@permission_required(ACCESS_PERM)
def deactivate_discord(request):
    logger.debug("deactivate_discord called by user %s", request.user)
    if request.user.discord.delete_user(
        is_rate_limited=False, handle_api_exceptions=True
    ):
        logger.info("Successfully deactivated discord for user %s", request.user)
        messages.success(request, _('Deactivated Discord account.'))
    else:
        logger.error(
            "Unsuccessful attempt to deactivate discord for user %s", request.user
        )
        messages.error(
            request, _('An error occurred while processing your Discord account.')
        )
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_discord(request):
    logger.debug("reset_discord called by user %s", request.user)
    if request.user.discord.delete_user(
        is_rate_limited=False, handle_api_exceptions=True
    ):
        logger.info(
            "Successfully deleted discord user for user %s - "
            "forwarding to discord activation.",
            request.user
        )
        return redirect("discord:activate")

    logger.error(
        "Unsuccessful attempt to reset discord for user %s", request.user
    )
    messages.error(
        request, _('An error occurred while processing your Discord account.')
    )
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def activate_discord(request):
    logger.debug("activate_discord called by user %s", request.user)
    return redirect(DiscordUser.objects.generate_oauth_redirect_url())


@login_required
@permission_required(ACCESS_PERM)
def discord_callback(request):
    logger.debug(
        "Received Discord callback for activation of user %s", request.user
    )
    authorization_code = request.GET.get('code', None)
    if not authorization_code:
        logger.warning(
            "Did not receive OAuth code from callback for user %s", request.user
        )
        success = False
    else:
        if DiscordUser.objects.add_user(
            user=request.user,
            authorization_code=authorization_code,
            is_rate_limited=False
        ):
            logger.info(
                "Successfully activated Discord account for user %s", request.user
            )
            success = True

        else:
            logger.error(
                "Failed to activate Discord account for user %s", request.user
            )
            success = False

    if success:
        messages.success(
            request, _('Your Discord account has been successfully activated.')
        )
    else:
        messages.error(
            request,
            _(
                'An error occurred while trying to activate your Discord account. '
                'Please try again.'
            )
        )

    return redirect("services:services")


@login_required
@user_passes_test(superuser_test)
def discord_add_bot(request):
    return redirect(DiscordUser.objects.generate_bot_add_url())



# Generated by Django 3.1.4 on 2020-12-30 08:53

from django.db import migrations


def add_aa_team_token(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Tokens = apps.get_model('analytics', 'AnalyticsTokens')
    token = Tokens()
    token.type = 'GA-U'
    token.token = 'UA-186249766-2'
    token.send_page_views = True
    token.send_celery_tasks = True
    token.send_stats = True
    token.name = 'AA Team Public Google Analytics (Universal)'
    token.save()


def remove_aa_team_token(apps, schema_editor):
    # Have to define some code to remove this identifier
    # In case of migration rollback?
    Tokens = apps.get_model('analytics', 'AnalyticsTokens')
    token = Tokens.objects.filter(token="UA-186249766-2").delete()


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0001_initial'),
        ]

    operations = [migrations.RunPython(add_aa_team_token, remove_aa_team_token)
                    ]



from collections import namedtuple
import datetime as dt

from .event_series import EventSeries


"""Global series for counting task events."""
succeeded_tasks = EventSeries("SUCCEEDED_TASKS")
retried_tasks = EventSeries("RETRIED_TASKS")
failed_tasks = EventSeries("FAILED_TASKS")


_TaskCounts = namedtuple(
    "_TaskCounts", ["succeeded", "retried", "failed", "total", "earliest_task", "hours"]
)


def dashboard_results(hours: int) -> _TaskCounts:
    """Counts of all task events within the given timeframe."""

    def earliest_if_exists(events: EventSeries, earliest: dt.datetime) -> list:
        my_earliest = events.first_event(earliest=earliest)
        return [my_earliest] if my_earliest else []

    earliest = dt.datetime.utcnow() - dt.timedelta(hours=hours)
    earliest_events = list()
    succeeded_count = succeeded_tasks.count(earliest=earliest)
    earliest_events += earliest_if_exists(succeeded_tasks, earliest)
    retried_count = retried_tasks.count(earliest=earliest)
    earliest_events += earliest_if_exists(retried_tasks, earliest)
    failed_count = failed_tasks.count(earliest=earliest)
    earliest_events += earliest_if_exists(failed_tasks, earliest)
    return _TaskCounts(
        succeeded=succeeded_count,
        retried=retried_count,
        failed=failed_count,
        total=succeeded_count + retried_count + failed_count,
        earliest_task=min(earliest_events) if earliest_events else None,
        hours=hours,
    )



class NotifyApiWrapper:
    """Wrapper to create notify API."""

    def __call__(self, *args, **kwargs):  # provide old API for backwards compatibility
        return self._add_notification(*args, **kwargs)

    def danger(self, user: object, title: str, message: str = None) -> None:
        """Add danger notification for user."""
        self._add_notification(user, title, message, level="danger")

    def info(self, user: object, title: str, message: str = None) -> None:
        """Add info notification for user."""
        self._add_notification(user=user, title=title, message=message, level="info")

    def success(self, user: object, title: str, message: str = None) -> None:
        """Add success notification for user."""
        self._add_notification(user, title, message, level="success")

    def warning(self, user: object, title: str, message: str = None) -> None:
        """Add warning notification for user."""
        self._add_notification(user, title, message, level="warning")

    def _add_notification(
        self, user: object, title: str, message: str = None, level: str = "info"
    ) -> None:
        from .models import Notification

        Notification.objects.notify_user(
            user=user, title=title, message=message, level=level
        )


notify = NotifyApiWrapper()



from django.conf.urls import url, include

from . import views

app_name = 'discord'

module_urls = [
    # Discord Service Control
    url(r'^activate/$', views.activate_discord, name='activate'),
    url(r'^deactivate/$', views.deactivate_discord, name='deactivate'),
    url(r'^reset/$', views.reset_discord, name='reset'),
    url(r'^callback/$', views.discord_callback, name='callback'),
    url(r'^add_bot/$', views.discord_add_bot, name='add_bot'),
]

urlpatterns = [
    url(r'^discord/', include((module_urls, app_name), namespace=app_name))
]



from typing import Set

from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.auth.models import User
from django.db import models
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from allianceauth.authentication.models import State
from allianceauth.notifications import notify


class GroupRequest(models.Model):
    """Request from a user for joining or leaving a group."""

    leave_request = models.BooleanField(default=0)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    def __str__(self):
        return self.user.username + ":" + self.group.name

    @property
    def main_char(self):
        """
        Legacy property for main character
        :return: self.users main character
        """
        return self.user.profile.main_character

    def notify_leaders(self) -> None:
        """Send notification to all group leaders about this request.

        Note: No translations, because language for each leader is unknown
        """
        if not getattr(settings, 'GROUPMANAGEMENT_REQUESTS_NOTIFICATION', False):
            return
        keyword = "leave" if self.leave_request else "join"
        title = f"Group Management: {keyword.title()} request for {self.group.name}"
        message = f"{self.user} want's to {keyword} {self.group.name}."
        for appover in self.group.authgroup.group_request_approvers():
            notify(user=appover, title=title, message=message, level="info")


class RequestLog(models.Model):
    """Log entry about who joined and left a group and who approved it."""

    request_type = models.BooleanField(null=True)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    request_info = models.CharField(max_length=254)
    action = models.BooleanField(default=0)
    request_actor = models.ForeignKey(User, on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now_add=True)

    def requestor(self):
        return self.request_info.split(":")[0]

    def type_to_str(self):
        if self.request_type is None:
            return "Removed"
        elif self.request_type is True:
            return "Leave"
        elif self.request_type is False:
            return "Join"

    def action_to_str(self):
        if self.action is True:
            return "Accept"
        elif self.action is False:
            return "Reject"

    def req_char(self):
        usr = self.requestor()
        user = User.objects.get(username=usr)
        return user.profile.main_character


class AuthGroup(models.Model):
    """
    Extends Django Group model with a one-to-one field
    Attributes are accessible via group as if they were in the model
    e.g. group.authgroup.internal

    Logic:
    Internal - not requestable by users, at all. Covers Corp_, Alliance_,
    Members etc groups. Groups are internal by default

    Public - Other options are respected, but any user will be able to become
    and remain a member, even if they have no API etc entered.
    Auth will not manage these groups automatically so user removal is up to
    group managers/leaders.

    Not Internal and:
        Hidden - users cannot view, can request if they have the direct link.
        Not Hidden - Users can view and request the group
        Open - Users are automatically accepted into the group
        Not Open - Users requests must be approved before they are added to the group
    """

    group = models.OneToOneField(Group, on_delete=models.CASCADE, primary_key=True)
    internal = models.BooleanField(
        default=True,
        help_text=_(
            "Internal group, users cannot see, join or request to join this group.<br>"
            "Used for groups such as Members, Corp_*, Alliance_* etc.<br>"
            "<b>Overrides Hidden and Open options when selected.</b>"
        )
    )
    hidden = models.BooleanField(
        default=True,
        help_text=_(
            "Group is hidden from users but can still join with the correct link."
        )
    )
    open = models.BooleanField(
        default=False,
        help_text=_(
            "Group is open and users will be automatically added upon request.<br>"
            "If the group is not open users will need their request manually approved."
        )
    )
    public = models.BooleanField(
        default=False,
        help_text=_(
            "Group is public. Any registered user is able to join this group, with "
            "visibility based on the other options set for this group.<br>"
            "Auth will not remove users from this group automatically when they "
            "are no longer authenticated."
        )
    )
    restricted = models.BooleanField(
        default=False,
        help_text=_(
            "Group is restricted. This means that adding or removing users "
            "for this group requires a superuser admin."
        )
    )
    group_leaders = models.ManyToManyField(
        User,
        related_name='leads_groups',
        blank=True,
        help_text=_(
            "Group leaders can process requests for this group. "
            "Use the <code>auth.group_management</code> permission to allow "
            "a user to manage all groups.<br>"
        )
    )
    group_leader_groups = models.ManyToManyField(
        Group,
        related_name='leads_group_groups',
        blank=True,
        help_text=_(
            "Members of leader groups can process requests for this group. "
            "Use the <code>auth.group_management</code> permission "
            "to allow a user to manage all groups.<br>")
    )
    states = models.ManyToManyField(
        State,
        related_name='valid_states',
        blank=True,
        help_text=_(
            "States listed here will have the ability to join this group provided "
            "they have the proper permissions.<br>"
        )
    )
    description = models.TextField(
        max_length=512,
        blank=True,
        help_text=_(
            "Short description <i>(max. 512 characters)</i> "
            "of the group shown to users."
        )
    )

    class Meta:
        permissions = (
            ("request_groups", _("Can request non-public groups")),
        )
        default_permissions = tuple()

    def __str__(self):
        return self.group.name

    def group_request_approvers(self) -> Set[User]:
        """Return all users who can approve a group request."""
        return set(
            self.group_leaders.all()
            | User.objects.filter(groups__in=list(self.group_leader_groups.all()))
        )


class ReservedGroupName(models.Model):
    """Name that can not be used for groups.

    This enables AA to ignore groups on other services (e.g. Discord) with that name.
    """

    name = models.CharField(
        _('name'),
        max_length=150,
        unique=True,
        help_text=_("Name that can not be used for groups.")
    )
    reason = models.TextField(
        _('reason'), help_text=_("Reason why this name is reserved.")
    )
    created_by = models.CharField(
        _('created by'),
        max_length=255,
        help_text="Name of the user who created this entry."
    )
    created_at = models.DateTimeField(
        _('created at'), default=now, help_text=_("Date when this entry was created")
    )

    def __str__(self) -> str:
        return self.name

    def save(self, *args, **kwargs) -> None:
        if Group.objects.filter(name__iexact=self.name).exists():
            raise RuntimeError(
                f"Save failed. There already exists a group with the name: {self.name}."
            )
        super().save(*args, **kwargs)



from allianceauth.analytics.middleware import AnalyticsMiddleware
from unittest.mock import Mock

from django.test.testcases import TestCase


class TestAnalyticsMiddleware(TestCase):

    def setUp(self):
        self.middleware = AnalyticsMiddleware()
        self.request = Mock()
        self.request.headers = {
                "User-Agent": "AUTOMATED TEST"
            }
        self.request.path = '/testURL/'
        self.request.session = {}
        self.request.LANGUAGE_CODE = 'en'
        self.response = Mock()
        self.response.content = 'hello world'

    def test_middleware(self):
        response = self.middleware.process_response(self.request, self.response)
        self.assertEqual(self.response, response)



from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from uuid import uuid4


class AnalyticsIdentifier(models.Model):

    identifier = models.UUIDField(default=uuid4,
                                  editable=False)

    def save(self, *args, **kwargs):
        if not self.pk and AnalyticsIdentifier.objects.exists():
            # Force a single object
            raise ValidationError('There is can be only one \
                                   AnalyticsIdentifier instance')
        self.pk = self.id = 1 # If this happens to be deleted and recreated, force it to be 1
        return super().save(*args, **kwargs)


class AnalyticsPath(models.Model):
    ignore_path = models.CharField(max_length=254, default="/example/", help_text="Regex Expression, If matched no Analytics Page View is sent")


class AnalyticsTokens(models.Model):

    class Analytics_Type(models.TextChoices):
        GA_U = 'GA-U', _('Google Analytics Universal')
        GA_V4 = 'GA-V4', _('Google Analytics V4')

    name = models.CharField(max_length=254)
    type = models.CharField(max_length=254, choices=Analytics_Type.choices)
    token = models.CharField(max_length=254, blank=False)
    send_page_views = models.BooleanField(default=False)
    send_celery_tasks = models.BooleanField(default=False)
    send_stats = models.BooleanField(default=False)
    ignore_paths = models.ManyToManyField(AnalyticsPath, blank=True)



from django import template

from allianceauth.hooks import get_hooks

register = template.Library()


def process_menu_items(hooks, request):
    _menu_items = list()
    items = [fn() for fn in hooks]
    items.sort(key=lambda i: i.order)
    for item in items:
        _menu_items.append(item.render(request))
    return _menu_items


@register.inclusion_tag('public/menublock.html', takes_context=True)
def menu_items(context):
    request = context['request']

    return {
        'menu_items': process_menu_items(get_hooks('menu_item_hook'), request),
    }



from logging import LogRecord, DEBUG

from django.contrib.auth.models import Permission, Group, User
from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils

from ..handlers import NotificationHandler
from ..models import Notification

MODULE_PATH = 'allianceauth.notifications.handlers'


class TestHandler(TestCase):
    def test_do_nothing_if_permission_does_not_exist(self):
        # given
        Permission.objects.get(codename="logging_notifications").delete()
        handler = NotificationHandler()
        record = LogRecord(
            name="name",
            level=DEBUG,
            pathname="pathname",
            lineno=42,
            msg="msg",
            args=[],
            exc_info=None,
            func="func"
        )
        # when
        handler.emit(record)
        # then
        self.assertEqual(Notification.objects.count(), 0)

    def test_should_emit_message_to_users_with_permission_only(self):
        # given
        AuthUtils.create_user('Lex Luthor')
        user_permission = AuthUtils.create_user('Bruce Wayne')
        user_permission = AuthUtils.add_permission_to_user_by_name(
            "auth.logging_notifications", user_permission
        )
        group = Group.objects.create(name="Dummy Group")
        perm = Permission.objects.get(codename="logging_notifications")
        group.permissions.add(perm)
        user_group = AuthUtils.create_user('Peter Parker')
        user_group.groups.add(group)
        user_superuser = User.objects.create_superuser("Clark Kent")
        handler = NotificationHandler()
        record = LogRecord(
            name="name",
            level=DEBUG,
            pathname="pathname",
            lineno=42,
            msg="msg",
            args=[],
            exc_info=None,
            func="func"
        )
        # when
        handler.emit(record)
        # then
        self.assertEqual(Notification.objects.count(), 3)
        users = set(Notification.objects.values_list("user__pk", flat=True))
        self.assertSetEqual(
            users, {user_permission.pk, user_group.pk, user_superuser.pk}
        )
        notif = Notification.objects.first()
        self.assertEqual(notif.user, user_permission)
        self.assertEqual(notif.title, "DEBUG [func:42]")
        self.assertEqual(notif.level, "success")
        self.assertEqual(notif.message, "msg")



# Generated by Django 1.10.1 on 2016-09-05 22:20

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('fleetactivitytracking', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fatlink',
            name='fatdatetime',
            field=models.DateTimeField(default=datetime.datetime(2016, 9, 5, 22, 20, 2, 999041, tzinfo=utc)),
        ),
    ]



import logging
import os

from allianceauth.authentication.models import CharacterOwnership, UserProfile
from bravado.exception import HTTPForbidden
from django.db import models
from esi.errors import TokenError
from esi.models import Token
from allianceauth.eveonline.models import EveCorporationInfo, EveCharacter, EveAllianceInfo
from allianceauth.notifications import notify

from allianceauth.corputils.managers import CorpStatsManager

SWAGGER_SPEC_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'swagger.json')
"""
Swagger spec operations:

Character
get_characters_character_id
get_corporations_corporation_id_members
Universe
post_universe_names
"""


logger = logging.getLogger(__name__)


class CorpStats(models.Model):
    token = models.ForeignKey(Token, on_delete=models.CASCADE)
    corp = models.OneToOneField(EveCorporationInfo, on_delete=models.CASCADE)
    last_update = models.DateTimeField(auto_now=True)

    class Meta:
        permissions = (
            ('view_corp_corpstats', 'Can view corp stats of their corporation.'),
            ('view_alliance_corpstats', 'Can view corp stats of members of their alliance.'),
            ('view_state_corpstats', 'Can view corp stats of members of their auth state.'),
        )
        verbose_name = "corp stats"
        verbose_name_plural = "corp stats"

    objects = CorpStatsManager()

    def __str__(self):
        return f"{self.__class__.__name__} for {self.corp}"

    def update(self):
        try:
            c = self.token.get_esi_client(spec_file=SWAGGER_SPEC_PATH)
            assert c.Character.get_characters_character_id(character_id=self.token.character_id).result()['corporation_id'] == int(self.corp.corporation_id)
            member_ids = c.Corporation.get_corporations_corporation_id_members(
                corporation_id=self.corp.corporation_id).result()

            # requesting too many ids per call results in a HTTP400
            # the swagger spec doesn't have a maxItems count
            # manual testing says we can do over 350, but let's not risk it
            member_id_chunks = [member_ids[i:i + 255] for i in range(0, len(member_ids), 255)]
            member_name_chunks = [c.Universe.post_universe_names(ids=id_chunk).result() for id_chunk in member_id_chunks]
            member_list = {}
            for name_chunk in member_name_chunks:
                member_list.update({m['id']: m['name'] for m in name_chunk})

            # bulk create new member models
            missing_members = [m_id for m_id in member_ids if not CorpMember.objects.filter(corpstats=self, character_id=m_id).exists()]
            CorpMember.objects.bulk_create(
                [CorpMember(character_id=m_id, character_name=member_list[m_id], corpstats=self) for m_id in missing_members])

            # purge old members
            self.members.exclude(character_id__in=member_ids).delete()

            # update the timer
            self.save()

        except TokenError as e:
            logger.warning(f"{self} failed to update: {e}")
            if self.token.user:
                notify(
                    self.token.user, "%s failed to update with your ESI token." % self,
                    message="Your token has expired or is no longer valid. Please add a new one to create a new CorpStats.",
                    level="error")
            self.delete()
        except HTTPForbidden as e:
            logger.warning(f"{self} failed to update: {e}")
            if self.token.user:
                notify(self.token.user, "%s failed to update with your ESI token." % self, message=f"{e.status_code}: {e.message}", level="error")
            self.delete()
        except AssertionError:
            logger.warning("%s token character no longer in corp." % self)
            if self.token.user:
                notify(
                    self.token.user, "%s cannot update with your ESI token." % self,
                    message="%s cannot update with your ESI token as you have left corp." % self, level="error")
            self.delete()

    @property
    def member_count(self):
        return self.members.count()

    @property
    def user_count(self):
        return len({m.main_character for m in self.members.all() if m.main_character})

    @property
    def registered_member_count(self):
        return len(self.registered_members)

    @property
    def registered_members(self):
        return self.members.filter(pk__in=[m.pk for m in self.members.all() if m.registered])

    @property
    def unregistered_member_count(self):
        return self.member_count - self.registered_member_count

    @property
    def unregistered_members(self):
        return self.members.filter(pk__in=[m.pk for m in self.members.all() if not m.registered])

    @property
    def main_count(self):
        return len(self.mains)

    @property
    def mains(self):
        return self.members.filter(pk__in=[m.pk for m in self.members.all() if m.main_character and int(m.main_character.character_id) == int(m.character_id)])

    def visible_to(self, user):
        return CorpStats.objects.filter(pk=self.pk).visible_to(user).exists()

    def can_update(self, user):
        return self.token.user == user or self.visible_to(user)

    def corp_logo(self, size=128):
        return self.corp.logo_url(size)

    def alliance_logo(self, size=128):
        if self.corp.alliance:
            return self.corp.alliance.logo_url(size)
        else:
            return EveAllianceInfo.generic_logo_url(1, size)


class CorpMember(models.Model):
    character_id = models.PositiveIntegerField()
    character_name = models.CharField(max_length=37)
    corpstats = models.ForeignKey(CorpStats, on_delete=models.CASCADE, related_name='members')

    class Meta:
        # not making character_id unique in case a character moves between two corps while only one updates
        unique_together = ('corpstats', 'character_id')
        ordering = ['character_name']

    def __str__(self):
        return self.character_name

    @property
    def character(self):
        try:
            return EveCharacter.objects.get(character_id=self.character_id)
        except EveCharacter.DoesNotExist:
            return None

    @property
    def main_character(self):
        try:
            return self.character.character_ownership.user.profile.main_character
        except (CharacterOwnership.DoesNotExist, UserProfile.DoesNotExist, AttributeError):
            return None

    @property
    def alts(self):
        if self.main_character:
            return [co.character for co in self.main_character.character_ownership.user.character_ownerships.all()]
        else:
            return []

    @property
    def registered(self):
        return CharacterOwnership.objects.filter(character__character_id=self.character_id).exists()

    def portrait_url(self, size=32):
        return EveCharacter.generic_portrait_url(self.character_id, size)

    @property
    def portrait_url_32(self):
        return self.portrait_url(32)

    @property
    def portrait_url_64(self):
        return self.portrait_url(64)

    @property
    def portrait_url_128(self):
        return self.portrait_url(128)



from . import views

from django.conf.urls import url

app_name = "groupmanagement"

urlpatterns = [
    # groups
    url(r"^groups/$", views.groups_view, name="groups"),
    url(r"^group/request/join/(\w+)/$", views.group_request_add, name="request_add"),
    url(
        r"^group/request/leave/(\w+)/$", views.group_request_leave, name="request_leave"
    ),
    # group management
    url(r"^groupmanagement/requests/$", views.group_management, name="management"),
    url(r"^groupmanagement/membership/$", views.group_membership, name="membership"),
    url(
        r"^groupmanagement/membership/(\w+)/$",
        views.group_membership_list,
        name="membership",
    ),
    url(
        r"^groupmanagement/membership/(\w+)/audit-log/$",
        views.group_membership_audit,
        name="audit_log",
    ),
    url(
        r"^groupmanagement/membership/(\w+)/remove/(\w+)/$",
        views.group_membership_remove,
        name="membership_remove",
    ),
    url(
        r"^groupmanagement/request/join/accept/(\w+)/$",
        views.group_accept_request,
        name="accept_request",
    ),
    url(
        r"^groupmanagement/request/join/reject/(\w+)/$",
        views.group_reject_request,
        name="reject_request",
    ),
    url(
        r"^groupmanagement/request/leave/accept/(\w+)/$",
        views.group_leave_accept_request,
        name="leave_accept_request",
    ),
    url(
        r"^groupmanagement/request/leave/reject/(\w+)/$",
        views.group_leave_reject_request,
        name="leave_reject_request",
    ),
]



from ..utils import clean_setting


# Base URL for all API calls. Must end with /.
DISCORD_API_BASE_URL = clean_setting(
    'DISCORD_API_BASE_URL', 'https://discord.com/api/'
)

# Low level connecttimeout for requests to the Discord API in seconds
DISCORD_API_TIMEOUT_CONNECT = clean_setting(
    'DISCORD_API_TIMEOUT', 5
)

# Low level read timeout for requests to the Discord API in seconds
DISCORD_API_TIMEOUT_READ = clean_setting(
    'DISCORD_API_TIMEOUT', 30
)

# Base authorization URL for Discord Oauth
DISCORD_OAUTH_BASE_URL = clean_setting(
    'DISCORD_OAUTH_BASE_URL', 'https://discord.com/api/oauth2/authorize'
)

# Base authorization URL for Discord Oauth
DISCORD_OAUTH_TOKEN_URL = clean_setting(
    'DISCORD_OAUTH_TOKEN_URL', 'https://discord.com/api/oauth2/token'
)

# How long the Discord guild names retrieved from the server are
# caches locally in seconds.
DISCORD_GUILD_NAME_CACHE_MAX_AGE = clean_setting(
    'DISCORD_GUILD_NAME_CACHE_MAX_AGE', 3600 * 24
)

# How long Discord roles retrieved from the server are caches locally in seconds.
DISCORD_ROLES_CACHE_MAX_AGE = clean_setting(
    'DISCORD_ROLES_CACHE_MAX_AGE', 3600 * 1
)

# Turns off creation of new roles. In case the rate limit for creating roles is
# exhausted, this setting allows the Discord service to continue to function
# and wait out the reset. Rate limit is about 250 per 48 hrs.
DISCORD_DISABLE_ROLE_CREATION = clean_setting(
    'DISCORD_DISABLE_ROLE_CREATION', False
)



import logging

from django.contrib import admin

from . import __title__
from ...admin import ServicesUserAdmin
from .models import DiscordUser
from .utils import LoggerAddTag


logger = LoggerAddTag(logging.getLogger(__name__), __title__)


@admin.register(DiscordUser)
class DiscordUserAdmin(ServicesUserAdmin):
    search_fields = ServicesUserAdmin.search_fields + ('uid', 'username')
    list_display = ServicesUserAdmin.list_display + ('activated', '_username', '_uid')
    list_filter = ServicesUserAdmin.list_filter + ('activated',)
    ordering = ('-activated',)

    def _uid(self, obj):
        return obj.uid

    _uid.short_description = 'Discord ID (UID)'
    _uid.admin_order_field = 'uid'

    def _username(self, obj):
        if obj.username and obj.discriminator:
            return f'{obj.username}#{obj.discriminator}'
        else:
            return ''

    def delete_queryset(self, request, queryset):
        for user in queryset:
            user.delete_user()

    _username.short_description = 'Discord Username'
    _username.admin_order_field = 'username'



from django.db import models

from allianceauth.authentication.models import State


class NameFormatConfig(models.Model):
    service_name = models.CharField(max_length=100, blank=False)
    default_to_username = models.BooleanField(
        default=True,
        help_text=
            'If a user has no main_character, '
            'default to using their Auth username instead.'
    )
    format = models.CharField(
        max_length=100,
        blank=False,
        help_text=
            'For information on constructing name formats '
            'please see the official documentation, '
            'topic "Services Name Formats".'
    )
    states = models.ManyToManyField(
        State,
        help_text=
            "States to apply this format to. You should only have one "
            "formatter for each state for each service."
    )

    def __str__(self):
        return '{}: {}'.format(
            self.service_name, ', '.join([str(x) for x in self.states.all()])
        )



import logging

from allianceauth.services.forms import ServicePasswordModelForm
from allianceauth.services.abstract import BaseCreatePasswordServiceAccountView, BaseDeactivateServiceAccountView, \
    BaseResetPasswordServiceAccountView, BaseSetPasswordServiceAccountView

from .models import MumbleUser

logger = logging.getLogger(__name__)


class MumblePasswordForm(ServicePasswordModelForm):
    class Meta:
        model = MumbleUser
        fields = ('password',)


class MumbleViewMixin:
    service_name = 'mumble'
    model = MumbleUser
    permission_required = 'mumble.access_mumble'


class CreateAccountMumbleView(MumbleViewMixin, BaseCreatePasswordServiceAccountView):
    pass


class DeleteMumbleView(MumbleViewMixin, BaseDeactivateServiceAccountView):
    pass


class ResetPasswordMumbleView(MumbleViewMixin, BaseResetPasswordServiceAccountView):
    pass


class SetPasswordMumbleView(MumbleViewMixin, BaseSetPasswordServiceAccountView):
    form_class = MumblePasswordForm



from django.apps import apps
from allianceauth.authentication.models import User
from esi.models import Token
from allianceauth.analytics.utils import install_stat_users, install_stat_tokens, install_stat_addons

from django.test.testcases import TestCase


def create_testdata():
    User.objects.all().delete()
    User.objects.create_user(
        'user_1'
        'abc@example.com',
        'password'
    )
    User.objects.create_user(
        'user_2'
        'abc@example.com',
        'password'
    )
    #Token.objects.all().delete()
    #Token.objects.create(
    #            character_id=101,
    #            character_name='character1',
    #            access_token='my_access_token'
    #)
    #Token.objects.create(
    #            character_id=102,
    #            character_name='character2',
    #            access_token='my_access_token'
    #)


class TestAnalyticsUtils(TestCase):

    def test_install_stat_users(self):
        create_testdata()
        expected = 2

        users = install_stat_users()
        self.assertEqual(users, expected)

    #def test_install_stat_tokens(self):
    #    create_testdata()
    #    expected = 2
    #
    #   tokens = install_stat_tokens()
    #   self.assertEqual(tokens, expected)

    def test_install_stat_addons(self):
        # this test does what its testing...
        # but helpful for existing as a sanity check
        expected = len(list(apps.get_app_configs()))
        addons = install_stat_addons()
        self.assertEqual(addons, expected)



from bs4 import BeautifulSoup
from urllib.parse import quote
from unittest.mock import patch, MagicMock

from django_webtest import WebTest

from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import Group
from django.test import TestCase, RequestFactory, Client

from allianceauth.authentication.models import (
    CharacterOwnership, State, OwnershipRecord
)
from allianceauth.eveonline.models import (
    EveCharacter, EveCorporationInfo, EveAllianceInfo, EveFactionInfo
)
from allianceauth.services.hooks import ServicesHook
from allianceauth.tests.auth_utils import AuthUtils

from ..admin import (
    BaseUserAdmin,
    CharacterOwnershipAdmin,
    StateAdmin,
    MainCorporationsFilter,
    MainAllianceFilter,
    MainFactionFilter,
    OwnershipRecordAdmin,
    User,
    UserAdmin,
    user_main_organization,
    user_profile_pic,
    user_username,
    update_main_character_model,
    make_service_hooks_update_groups_action,
    make_service_hooks_sync_nickname_action
)
from . import get_admin_change_view_url, get_admin_search_url


MODULE_PATH = 'allianceauth.authentication.admin'


class MockRequest:
    def __init__(self, user=None):
        self.user = user


class TestCaseWithTestData(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        for MyModel in [
            EveAllianceInfo, EveCorporationInfo, EveCharacter, Group, User
        ]:
            MyModel.objects.all().delete()

        # groups
        cls.group_1 = Group.objects.create(
            name='Group 1'
        )
        cls.group_2 = Group.objects.create(
            name='Group 2'
        )

        # user 1 - corp and alliance, normal user
        character_1 = EveCharacter.objects.create(
            character_id=1001,
            character_name='Bruce Wayne',
            corporation_id=2001,
            corporation_name='Wayne Technologies',
            corporation_ticker='WT',
            alliance_id=3001,
            alliance_name='Wayne Enterprises',
            alliance_ticker='WE',
        )
        character_1a = EveCharacter.objects.create(
            character_id=1002,
            character_name='Batman',
            corporation_id=2001,
            corporation_name='Wayne Technologies',
            corporation_ticker='WT',
            alliance_id=3001,
            alliance_name='Wayne Enterprises',
            alliance_ticker='WE',
        )
        alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Wayne Enterprises',
            alliance_ticker='WE',
            executor_corp_id=2001
        )
        EveCorporationInfo.objects.create(
            corporation_id=2001,
            corporation_name='Wayne Technologies',
            corporation_ticker='WT',
            member_count=42,
            alliance=alliance
        )
        cls.user_1 = User.objects.create_user(
            character_1.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=character_1,
            owner_hash='x1' + character_1.character_name,
            user=cls.user_1
        )
        CharacterOwnership.objects.create(
            character=character_1a,
            owner_hash='x1' + character_1a.character_name,
            user=cls.user_1
        )
        cls.user_1.profile.main_character = character_1
        cls.user_1.profile.save()
        cls.user_1.groups.add(cls.group_1)

        # user 2 - corp only, staff
        character_2 = EveCharacter.objects.create(
            character_id=1003,
            character_name='Clark Kent',
            corporation_id=2002,
            corporation_name='Daily Planet',
            corporation_ticker='DP',
            alliance_id=None
        )
        EveCorporationInfo.objects.create(
            corporation_id=2002,
            corporation_name='Daily Plane',
            corporation_ticker='DP',
            member_count=99,
            alliance=None
        )
        cls.user_2 = User.objects.create_user(
            character_2.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=character_2,
            owner_hash='x1' + character_2.character_name,
            user=cls.user_2
        )
        cls.user_2.profile.main_character = character_2
        cls.user_2.profile.save()
        cls.user_2.groups.add(cls.group_2)
        cls.user_2.is_staff = True
        cls.user_2.save()

        # user 3 - no main, no group, superuser
        character_3 = EveCharacter.objects.create(
            character_id=1101,
            character_name='Lex Luthor',
            corporation_id=2101,
            corporation_name='Lex Corp',
            corporation_ticker='LC',
            alliance_id=None
        )
        EveCorporationInfo.objects.create(
            corporation_id=2101,
            corporation_name='Lex Corp',
            corporation_ticker='LC',
            member_count=666,
            alliance=None
        )
        EveAllianceInfo.objects.create(
            alliance_id=3101,
            alliance_name='Lex World Domination',
            alliance_ticker='LWD',
            executor_corp_id=2101
        )
        cls.user_3 = User.objects.create_user(
            character_3.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=character_3,
            owner_hash='x1' + character_3.character_name,
            user=cls.user_3
        )
        cls.user_3.is_superuser = True
        cls.user_3.save()

        # user 4 - corp and faction, no alliance
        cls.character_4 = EveCharacter.objects.create(
            character_id=4321,
            character_name='Professor X',
            corporation_id=5432,
            corporation_name="Xavier's School for Gifted Youngsters",
            corporation_ticker='MUTNT',
            alliance_id=None,
            faction_id=999,
            faction_name='The X-Men',
        )
        cls.user_4 = User.objects.create_user(
            cls.character_4.character_name.replace(' ', '_'),
            'abc@example.com',
            'password'
        )
        CharacterOwnership.objects.create(
            character=cls.character_4,
            owner_hash='x1' + cls.character_4.character_name,
            user=cls.user_4
        )
        cls.user_4.profile.main_character = cls.character_4
        cls.user_4.profile.save()
        EveFactionInfo.objects.create(faction_id=999, faction_name='The X-Men')


def make_generic_search_request(ModelClass: type, search_term: str):
    User.objects.create_superuser(
        username='superuser', password='secret', email='admin@example.com'
    )
    c = Client()
    c.login(username='superuser', password='secret')
    return c.get(
        f'{get_admin_search_url(ModelClass)}?q={quote(search_term)}'
    )


class TestCharacterOwnershipAdmin(TestCaseWithTestData):
    fixtures = ["disable_analytics"]

    def setUp(self):
        self.modeladmin = CharacterOwnershipAdmin(
            model=User, admin_site=AdminSite()
        )

    def test_change_view_loads_normally(self):
        User.objects.create_superuser(
            username='superuser', password='secret', email='admin@example.com'
        )
        c = Client()
        c.login(username='superuser', password='secret')
        ownership = self.user_1.character_ownerships.first()
        response = c.get(get_admin_change_view_url(ownership))
        self.assertEqual(response.status_code, 200)

    def test_search_works(self):
        obj = CharacterOwnership.objects\
            .filter(user=self.user_1)\
            .first()
        response = make_generic_search_request(type(obj), obj.user.username)
        expected = 200
        self.assertEqual(response.status_code, expected)


class TestOwnershipRecordAdmin(TestCaseWithTestData):
    fixtures = ["disable_analytics"]

    def setUp(self):
        self.modeladmin = OwnershipRecordAdmin(
            model=User, admin_site=AdminSite()
        )

    def test_change_view_loads_normally(self):
        User.objects.create_superuser(
            username='superuser', password='secret', email='admin@example.com'
        )
        c = Client()
        c.login(username='superuser', password='secret')
        ownership_record = OwnershipRecord.objects\
            .filter(user=self.user_1)\
            .first()
        response = c.get(get_admin_change_view_url(ownership_record))
        self.assertEqual(response.status_code, 200)

    def test_search_works(self):
        obj = OwnershipRecord.objects.first()
        response = make_generic_search_request(type(obj), obj.user.username)
        expected = 200
        self.assertEqual(response.status_code, expected)


class TestStateAdmin(TestCaseWithTestData):
    fixtures = ["disable_analytics"]

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.modeladmin = StateAdmin(model=User, admin_site=AdminSite())

    def test_change_view_loads_normally(self):
        User.objects.create_superuser(
            username='superuser', password='secret', email='admin@example.com'
        )
        c = Client()
        c.login(username='superuser', password='secret')

        guest_state = AuthUtils.get_guest_state()
        response = c.get(get_admin_change_view_url(guest_state))
        self.assertEqual(response.status_code, 200)

        member_state = AuthUtils.get_member_state()
        response = c.get(get_admin_change_view_url(member_state))
        self.assertEqual(response.status_code, 200)

    def test_search_works(self):
        obj = State.objects.first()
        response = make_generic_search_request(type(obj), obj.name)
        expected = 200
        self.assertEqual(response.status_code, expected)


class TestUserAdmin(TestCaseWithTestData):
    fixtures = ["disable_analytics"]

    def setUp(self):
        self.factory = RequestFactory()
        self.modeladmin = UserAdmin(
            model=User, admin_site=AdminSite()
        )
        self.character_1 = self.user_1.character_ownerships.first().character

    def test_user_profile_pic_u1(self):
        expected = (
            '<img src="https://images.evetech.net/characters/1001/'
            'portrait?size=32" class="img-circle">'
        )
        self.assertEqual(user_profile_pic(self.user_1), expected)

    def test_user_profile_pic_u3(self):
        self.assertIsNone(user_profile_pic(self.user_3))

    def test_user_username_u1(self):
        expected = (
            '<strong><a href="/admin/authentication/user/{}/change/">'
            'Bruce_Wayne</a></strong><br>Bruce Wayne'.format(self.user_1.pk)
        )
        self.assertEqual(user_username(self.user_1), expected)

    def test_user_username_u3(self):
        expected = (
            '<strong><a href="/admin/authentication/user/{}/change/">'
            'Lex_Luthor</a></strong>'.format(self.user_3.pk)
        )
        self.assertEqual(user_username(self.user_3), expected)

    def test_user_main_organization_u1(self):
        expected = 'Wayne Technologies<br>Wayne Enterprises'
        self.assertEqual(user_main_organization(self.user_1), expected)

    def test_user_main_organization_u2(self):
        expected = 'Daily Planet'
        self.assertEqual(user_main_organization(self.user_2), expected)

    def test_user_main_organization_u3(self):
        expected = ''
        self.assertEqual(user_main_organization(self.user_3), expected)

    def test_user_main_organization_u4(self):
        expected = "Xavier's School for Gifted Youngsters<br>The X-Men"
        self.assertEqual(user_main_organization(self.user_4), expected)

    def test_characters_u1(self):
        expected = 'Batman, Bruce Wayne'
        result = self.modeladmin._characters(self.user_1)
        self.assertEqual(result, expected)

    def test_characters_u2(self):
        expected = 'Clark Kent'
        result = self.modeladmin._characters(self.user_2)
        self.assertEqual(result, expected)

    def test_characters_u3(self):
        expected = 'Lex Luthor'
        result = self.modeladmin._characters(self.user_3)
        self.assertEqual(result, expected)

    def test_groups_u1(self):
        expected = 'Group 1'
        result = self.modeladmin._groups(self.user_1)
        self.assertEqual(result, expected)

    def test_groups_u2(self):
        expected = 'Group 2'
        result = self.modeladmin._groups(self.user_2)
        self.assertEqual(result, expected)

    def test_groups_u3(self):
        result = self.modeladmin._groups(self.user_3)
        self.assertIsNone(result)

    def test_state(self):
        expected = 'Guest'
        result = self.modeladmin._state(self.user_1)
        self.assertEqual(result, expected)

    def test_role_u1(self):
        expected = 'User'
        result = self.modeladmin._role(self.user_1)
        self.assertEqual(result, expected)

    def test_role_u2(self):
        expected = 'Staff'
        result = self.modeladmin._role(self.user_2)
        self.assertEqual(result, expected)

    def test_role_u3(self):
        expected = 'Superuser'
        result = self.modeladmin._role(self.user_3)
        self.assertEqual(result, expected)

    def test_list_2_html_w_tooltips_no_cutoff(self):
        items = ['one', 'two', 'three']
        expected = 'one, two, three'
        result = self.modeladmin._list_2_html_w_tooltips(items, 5)
        self.assertEqual(expected, result)

    def test_list_2_html_w_tooltips_w_cutoff(self):
        items = ['one', 'two', 'three']
        expected = (
            '<span data-tooltip="one, two, three" '
            'class="tooltip">one, two, (...)</span>'
        )
        result = self.modeladmin._list_2_html_w_tooltips(items, 2)
        self.assertEqual(expected, result)

    def test_list_2_html_w_tooltips_empty_list(self):
        items = []
        expected = None
        result = self.modeladmin._list_2_html_w_tooltips(items, 5)
        self.assertEqual(expected, result)

    # actions

    @patch(MODULE_PATH + '.UserAdmin.message_user', auto_spec=True, unsafe=True)
    @patch(MODULE_PATH + '.update_character')
    def test_action_update_main_character_model(
        self, mock_task, mock_message_user
    ):
        users_qs = User.objects.filter(pk__in=[self.user_1.pk, self.user_2.pk])
        update_main_character_model(
            self.modeladmin, MockRequest(self.user_1), users_qs
        )
        self.assertEqual(mock_task.delay.call_count, 2)
        self.assertTrue(mock_message_user.called)

    # filters

    def test_filter_main_corporations(self):

        class UserAdminTest(BaseUserAdmin):
            list_filter = (MainCorporationsFilter,)

        my_modeladmin = UserAdminTest(User, AdminSite())

        # Make sure the lookups are correct
        request = self.factory.get('/')
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        filters = changelist.get_filters(request)
        filterspec = filters[0][0]
        expected = [
            (2002, 'Daily Planet'),
            (2001, 'Wayne Technologies'),
            (5432, "Xavier's School for Gifted Youngsters"),
        ]
        self.assertEqual(filterspec.lookup_choices, expected)

        # Make sure the correct queryset is returned
        request = self.factory.get(
            '/',
            {'main_corporation_id__exact': self.character_1.corporation_id}
        )
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        queryset = changelist.get_queryset(request)
        expected = [self.user_1]
        self.assertSetEqual(set(queryset), set(expected))

    def test_filter_main_alliances(self):

        class UserAdminTest(BaseUserAdmin):
            list_filter = (MainAllianceFilter,)

        my_modeladmin = UserAdminTest(User, AdminSite())

        # Make sure the lookups are correct
        request = self.factory.get('/')
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        filters = changelist.get_filters(request)
        filterspec = filters[0][0]
        expected = [
            (3001, 'Wayne Enterprises'),
        ]
        self.assertEqual(filterspec.lookup_choices, expected)

        # Make sure the correct queryset is returned
        request = self.factory.get(
            '/',
            {'main_alliance_id__exact': self.character_1.alliance_id}
        )
        request.user = self.user_1
        changelist = my_modeladmin.get_changelist_instance(request)
        queryset = changelist.get_queryset(request)
        expected = [self.user_1]
        self.assertSetEqual(set(queryset), set(expected))

    def test_filter_main_factions(self):
        class UserAdminTest(BaseUserAdmin):
            list_filter = (MainFactionFilter,)

        my_modeladmin = UserAdminTest(User, AdminSite())

        # Make sure the lookups are correct
        request = self.factory.get('/')
        request.user = self.user_4
        changelist = my_modeladmin.get_changelist_instance(request)
        filters = changelist.get_filters(request)
        filterspec = filters[0][0]
        expected = [
            (999, 'The X-Men'),
        ]
        self.assertEqual(filterspec.lookup_choices, expected)

        # Make sure the correct queryset is returned
        request = self.factory.get(
            '/',
            {'main_faction_id__exact': self.character_4.faction_id}
        )
        request.user = self.user_4
        changelist = my_modeladmin.get_changelist_instance(request)
        queryset = changelist.get_queryset(request)
        expected = [self.user_4]
        self.assertSetEqual(set(queryset), set(expected))

    def test_change_view_loads_normally(self):
        User.objects.create_superuser(
            username='superuser', password='secret', email='admin@example.com'
        )
        c = Client()
        c.login(username='superuser', password='secret')
        response = c.get(get_admin_change_view_url(self.user_1))
        self.assertEqual(response.status_code, 200)

    def test_search_works(self):
        obj = User.objects.first()
        response = make_generic_search_request(type(obj), obj.username)
        expected = 200
        self.assertEqual(response.status_code, expected)


class TestStateAdminChangeFormSuperuserExclusiveEdits(WebTest):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.super_admin = User.objects.create_superuser("super_admin")
        cls.staff_admin = User.objects.create_user("staff_admin")
        cls.staff_admin.is_staff = True
        cls.staff_admin.save()
        cls.staff_admin = AuthUtils.add_permissions_to_user_by_name(
            [
                "authentication.add_state",
                "authentication.change_state",
                "authentication.view_state",
            ],
            cls.staff_admin
        )
        cls.superuser_exclusive_fields = ["permissions",]

    def test_should_show_all_fields_to_superuser_for_add(self):
        # given
        self.app.set_user(self.super_admin)
        page = self.app.get("/admin/authentication/state/add/")
        # when
        form = page.forms["state_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertIn(field, form.fields)

    def test_should_not_show_all_fields_to_staff_admins_for_add(self):
        # given
        self.app.set_user(self.staff_admin)
        page = self.app.get("/admin/authentication/state/add/")
        # when
        form = page.forms["state_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertNotIn(field, form.fields)

    def test_should_show_all_fields_to_superuser_for_change(self):
        # given
        self.app.set_user(self.super_admin)
        state = AuthUtils.get_member_state()
        page = self.app.get(f"/admin/authentication/state/{state.pk}/change/")
        # when
        form = page.forms["state_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertIn(field, form.fields)

    def test_should_not_show_all_fields_to_staff_admin_for_change(self):
        # given
        self.app.set_user(self.staff_admin)
        state = AuthUtils.get_member_state()
        page = self.app.get(f"/admin/authentication/state/{state.pk}/change/")
        # when
        form = page.forms["state_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertNotIn(field, form.fields)


class TestUserAdminChangeForm(TestCase):
    fixtures = ["disable_analytics"]

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.modeladmin = UserAdmin(model=User, admin_site=AdminSite())

    def test_should_show_groups_available_to_user_with_blue_state_only(self):
        # given
        superuser = User.objects.create_superuser("Super")
        user = AuthUtils.create_user("bruce_wayne")
        character = AuthUtils.add_main_character_2(
            user,
            name="Bruce Wayne",
            character_id=1001,
            corp_id=2001,
            corp_name="Wayne Technologies"
        )
        blue_state = State.objects.get(name="Blue")
        blue_state.member_characters.add(character)
        member_state = AuthUtils.get_member_state()
        group_1 = Group.objects.create(name="Group 1")
        group_2 = Group.objects.create(name="Group 2")
        group_2.authgroup.states.add(blue_state)
        group_3 = Group.objects.create(name="Group 3")
        group_3.authgroup.states.add(member_state)
        self.client.force_login(superuser)
        # when
        response = self.client.get(f"/admin/authentication/user/{user.pk}/change/")
        # then
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.rendered_content, features="html.parser")
        groups_select = soup.find("select", {"id": "id_groups"}).find_all('option')
        group_ids = {int(option["value"]) for option in groups_select}
        self.assertSetEqual(group_ids, {group_1.pk, group_2.pk})


class TestUserAdminChangeFormSuperuserExclusiveEdits(WebTest):
    fixtures = ["disable_analytics"]

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.super_admin = User.objects.create_superuser("super_admin")
        cls.staff_admin = User.objects.create_user("staff_admin")
        cls.staff_admin.is_staff = True
        cls.staff_admin.save()
        cls.staff_admin = AuthUtils.add_permissions_to_user_by_name(
            [
                "auth.change_user",
                "auth.view_user",
                "authentication.change_user",
                "authentication.change_userprofile",
                "authentication.view_user"
            ],
            cls.staff_admin
        )
        cls.superuser_exclusive_fields = [
            "is_staff", "is_superuser", "user_permissions"
        ]

    def setUp(self) -> None:
        self.user = AuthUtils.create_user("bruce_wayne")

    def test_should_show_all_fields_to_superuser_for_change(self):
        # given
        self.app.set_user(self.super_admin)

        page = self.app.get(f"/admin/authentication/user/{self.user.pk}/change/")
        # when
        form = page.forms["user_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertIn(field, form.fields)

    def test_should_not_show_all_fields_to_staff_admin_for_change(self):
        # given
        self.app.set_user(self.staff_admin)
        page = self.app.get(f"/admin/authentication/user/{self.user.pk}/change/")
        # when
        form = page.forms["user_form"]
        # then
        for field in self.superuser_exclusive_fields:
            with self.subTest(field=field):
                self.assertNotIn(field, form.fields)

    def test_should_allow_super_admin_to_add_restricted_group_to_user(self):
        # given
        self.app.set_user(self.super_admin)
        group_restricted = Group.objects.create(name="restricted group")
        group_restricted.authgroup.restricted = True
        group_restricted.authgroup.save()
        page = self.app.get(f"/admin/authentication/user/{self.user.pk}/change/")
        form = page.forms["user_form"]
        # when
        form["groups"].select_multiple(texts=["restricted group"])
        response = form.submit("save")
        # then
        self.assertEqual(response.status_code, 302)
        self.user.refresh_from_db()
        self.assertIn(
            "restricted group", self.user.groups.values_list("name", flat=True)
        )

    def test_should_not_allow_staff_admin_to_add_restricted_group_to_user(self):
        # given
        self.app.set_user(self.staff_admin)
        group_restricted = Group.objects.create(name="restricted group")
        group_restricted.authgroup.restricted = True
        group_restricted.authgroup.save()
        page = self.app.get(f"/admin/authentication/user/{self.user.pk}/change/")
        form = page.forms["user_form"]
        # when
        form["groups"].select_multiple(texts=["restricted group"])
        response = form.submit("save")
        # then
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            "You are not allowed to add or remove these restricted groups",
            response.text
        )

    def test_should_not_allow_staff_admin_to_remove_restricted_group_from_user(self):
        # given
        self.app.set_user(self.staff_admin)
        group_restricted = Group.objects.create(name="restricted group")
        group_restricted.authgroup.restricted = True
        group_restricted.authgroup.save()
        self.user.groups.add(group_restricted)
        page = self.app.get(f"/admin/authentication/user/{self.user.pk}/change/")
        form = page.forms["user_form"]
        # when
        form["groups"].select_multiple(texts=[])
        response = form.submit("save")
        # then
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            "You are not allowed to add or remove these restricted groups",
            response.text
        )

    def test_should_allow_staff_admin_to_add_normal_group_to_user(self):
        # given
        self.app.set_user(self.super_admin)
        Group.objects.create(name="normal group")
        page = self.app.get(f"/admin/authentication/user/{self.user.pk}/change/")
        form = page.forms["user_form"]
        # when
        form["groups"].select_multiple(texts=["normal group"])
        response = form.submit("save")
        # then
        self.assertEqual(response.status_code, 302)
        self.user.refresh_from_db()
        self.assertIn("normal group", self.user.groups.values_list("name", flat=True))


class TestMakeServicesHooksActions(TestCaseWithTestData):

    class MyServicesHookTypeA(ServicesHook):

        def __init__(self):
            super().__init__()
            self.name = 'My Service A'

        def update_groups(self, user):
            pass

        def sync_nicknames(self, user):
            pass

    class MyServicesHookTypeB(ServicesHook):

        def __init__(self):
            super().__init__()
            self.name = 'My Service B'

        def update_groups(self, user):
            pass

        def update_groups_bulk(self, user):
            pass

        def sync_nicknames(self, user):
            pass

        def sync_nicknames_bulk(self, user):
            pass

    def test_service_has_update_groups_only(self):
        service = self.MyServicesHookTypeA()
        mock_service = MagicMock(spec=service)
        action = make_service_hooks_update_groups_action(mock_service)
        action(MagicMock(), MagicMock(), [self.user_1])
        self.assertTrue(mock_service.update_groups.called)

    def test_service_has_update_groups_bulk(self):
        service = self.MyServicesHookTypeB()
        mock_service = MagicMock(spec=service)
        action = make_service_hooks_update_groups_action(mock_service)
        action(MagicMock(), MagicMock(), [self.user_1])
        self.assertFalse(mock_service.update_groups.called)
        self.assertTrue(mock_service.update_groups_bulk.called)

    def test_service_has_sync_nickname_only(self):
        service = self.MyServicesHookTypeA()
        mock_service = MagicMock(spec=service)
        action = make_service_hooks_sync_nickname_action(mock_service)
        action(MagicMock(), MagicMock(), [self.user_1])
        self.assertTrue(mock_service.sync_nickname.called)

    def test_service_has_sync_nicknames_bulk(self):
        service = self.MyServicesHookTypeB()
        mock_service = MagicMock(spec=service)
        action = make_service_hooks_sync_nickname_action(mock_service)
        action(MagicMock(), MagicMock(), [self.user_1])
        self.assertFalse(mock_service.sync_nickname.called)
        self.assertTrue(mock_service.sync_nicknames_bulk.called)



import re

from django import forms
from django.utils.translation import ugettext_lazy as _


class SrpFleetMainForm(forms.Form):
    fleet_name = forms.CharField(required=True, label=_("Fleet Name"))
    fleet_time = forms.DateTimeField(required=True, label=_("Fleet Time"))
    fleet_doctrine = forms.CharField(required=True, label=_("Fleet Doctrine"))


class SrpFleetUserRequestForm(forms.Form):
    additional_info = forms.CharField(required=False, max_length=25, label=_("Additional Info"))
    killboard_link = forms.CharField(
        label=_("Killboard Link (zkillboard.com or kb.evetools.org)"),
        max_length=255,
        required=True

    )

    def clean_killboard_link(self):
        data = self.cleaned_data['killboard_link']

        # Check if it's a link from one of the accepted kill boards
        if not any(
            re.match(regex, data)
            for regex in [
                r"^http[s]?:\/\/zkillboard\.com\/",
                r"^http[s]?:\/\/kb\.evetools\.org\/",
            ]
        ):
            raise forms.ValidationError(
                _("Invalid Link. Please use zkillboard.com or kb.evetools.org")
            )

        # Check if it's an actual kill mail
        if not any(
            re.match(regex, data)
            for regex in [
                r"^http[s]?:\/\/zkillboard\.com\/kill\/\d+\/",
                r"^http[s]?:\/\/kb\.evetools\.org\/kill\/\d+",
            ]
        ):
            raise forms.ValidationError(
                _("Invalid Link. Please post a direct link to a killmail.")
            )

        return data


class SrpFleetMainUpdateForm(forms.Form):
    fleet_aar_link = forms.CharField(required=True, label=_("After Action Report Link"))



from unittest.mock import patch

from django.conf import settings
from django.contrib.auth.models import User
from django.test import TestCase, override_settings

from allianceauth.tests.auth_utils import AuthUtils
from ..models import Notification


MODULE_PATH = 'allianceauth.notifications.models'

NOTIFICATIONS_MAX_PER_USER_DEFAULT = 42


class TestQuerySet(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user_1 = AuthUtils.create_user('Peter Parker')
        cls.user_2 = AuthUtils.create_user('Clark Kent')

    @patch(MODULE_PATH + '.Notification.objects.invalidate_user_notification_cache')
    def test_update_will_invalidate_cache(
        self, mock_invalidate_user_notification_cache
    ):
        Notification.objects.notify_user(self.user_1, 'dummy_1')
        Notification.objects.notify_user(self.user_2, 'dummy_2')
        Notification.objects.update(viewed=True)
        self.assertEqual(mock_invalidate_user_notification_cache.call_count, 2)


class TestUserNotify(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('magic_mike')
        AuthUtils.add_main_character(
            cls.user,
            'Magic Mike',
            '1',
            corp_id='2',
            corp_name='Pole Riders',
            corp_ticker='PRIDE',
            alliance_id='3',
            alliance_name='RIDERS'
        )

    def test_can_notify(self):
        title = 'dummy_title'
        message = 'dummy message'
        level = 'danger'
        Notification.objects.notify_user(self.user, title, message, level)
        self.assertEqual(Notification.objects.filter(user=self.user).count(), 1)
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, title)
        self.assertEqual(obj.message, message)
        self.assertEqual(obj.level, level)

    def test_use_message_as_title_if_missing(self):
        title = 'dummy_title'
        Notification.objects.notify_user(self.user, title)
        self.assertEqual(Notification.objects.filter(user=self.user).count(), 1)
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, title)
        self.assertEqual(obj.message, title)

    def test_should_use_default_level_when_not_specified(self):
        # given
        title = 'dummy_title'
        message = 'dummy message'
        # when
        Notification.objects.notify_user(self.user, title, message)
        # then
        self.assertEqual(Notification.objects.filter(user=self.user).count(), 1)
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, title)
        self.assertEqual(obj.message, message)
        self.assertEqual(obj.level, Notification.Level.INFO)

    def test_should_use_default_level_when_invalid_level_given(self):
        # given
        title = 'dummy_title'
        message = 'dummy message'
        level = "invalid"
        # when
        Notification.objects.notify_user(self.user, title, message, level)
        # then
        self.assertEqual(Notification.objects.filter(user=self.user).count(), 1)
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, title)
        self.assertEqual(obj.message, message)
        self.assertEqual(obj.level, Notification.Level.INFO)

    @override_settings(NOTIFICATIONS_MAX_PER_USER=3)
    def test_remove_when_too_many_notifications(self):
        Notification.objects.notify_user(self.user, 'dummy')
        obj_2 = Notification.objects.notify_user(self.user, 'dummy')
        obj_3 = Notification.objects.notify_user(self.user, 'dummy')
        obj_4 = Notification.objects.notify_user(self.user, 'dummy')
        expected = {obj_2.pk, obj_3.pk, obj_4.pk}
        result = set(
            Notification.objects.filter(user=self.user).values_list("pk", flat=True)
        )
        self.assertSetEqual(result, expected)
        obj_5 = Notification.objects.notify_user(self.user, 'dummy')
        expected = {obj_3.pk, obj_4.pk, obj_5.pk}
        result = set(
            Notification.objects.filter(user=self.user).values_list("pk", flat=True)
        )
        self.assertSetEqual(result, expected)


@patch("allianceauth.notifications.managers.logger")
@patch(
    MODULE_PATH + ".Notification.NOTIFICATIONS_MAX_PER_USER_DEFAULT",
    NOTIFICATIONS_MAX_PER_USER_DEFAULT
)
class TestMaxNotificationsPerUser(TestCase):
    @override_settings(NOTIFICATIONS_MAX_PER_USER=42)
    def test_should_use_custom_integer_setting(self, mock_logger):
        # when
        result = Notification.objects._max_notifications_per_user()
        # then
        self.assertEqual(result, 42)
        self.assertFalse(mock_logger.warning.called)

    @override_settings(NOTIFICATIONS_MAX_PER_USER="42")
    def test_should_use_custom_string_setting(self, mock_logger):
        # when
        result = Notification.objects._max_notifications_per_user()
        # then
        self.assertEqual(result, 42)
        self.assertFalse(mock_logger.warning.called)

    @override_settings()
    def test_should_use_default_if_not_defined(self, mock_logger):
        # given
        del settings.NOTIFICATIONS_MAX_PER_USER
        # when
        result = Notification.objects._max_notifications_per_user()
        # then
        self.assertEqual(result, NOTIFICATIONS_MAX_PER_USER_DEFAULT)
        self.assertFalse(mock_logger.warning.called)

    @override_settings(NOTIFICATIONS_MAX_PER_USER="abc")
    def test_should_reset_to_default_if_not_int(self, mock_logger):
        # when
        result = Notification.objects._max_notifications_per_user()
        # then
        self.assertEqual(result, NOTIFICATIONS_MAX_PER_USER_DEFAULT)
        self.assertTrue(mock_logger.warning.called)

    @override_settings(NOTIFICATIONS_MAX_PER_USER=-1)
    def test_should_reset_to_default_if_lt_zero(self, mock_logger):
        # when
        result = Notification.objects._max_notifications_per_user()
        # then
        self.assertEqual(result, NOTIFICATIONS_MAX_PER_USER_DEFAULT)
        self.assertTrue(mock_logger.warning.called)


@patch('allianceauth.notifications.managers.cache')
class TestUnreadCount(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user_1 = AuthUtils.create_user('magic_mike')
        AuthUtils.add_main_character(
            cls.user_1,
            'Magic Mike',
            '1',
            corp_id='2',
            corp_name='Pole Riders',
            corp_ticker='PRIDE',
            alliance_id='3',
            alliance_name='RIDERS'
        )

        # test notifications for mike
        Notification.objects.all().delete()
        Notification.objects.create(
            user=cls.user_1,
            level="INFO",
            title="Job 1 Failed",
            message="Because it was broken",
            viewed=True
        )
        Notification.objects.create(
            user=cls.user_1,
            level="INFO",
            title="Job 2 Failed",
            message="Because it was broken"
        )
        Notification.objects.create(
            user=cls.user_1,
            level="INFO",
            title="Job 3 Failed",
            message="Because it was broken"
        )

        cls.user_2 = AuthUtils.create_user('teh_kid')
        AuthUtils.add_main_character(
            cls.user_2,
            'The Kid', '2',
            corp_id='2',
            corp_name='Pole Riders',
            corp_ticker='PRIDE',
            alliance_id='3',
            alliance_name='RIDERS'
        )

        # Notifications for kid
        Notification.objects.create(
            user=cls.user_2,
            level="INFO",
            title="Job 6 Failed",
            message="Because it was broken"
        )

    def test_update_cache_when_not_in_cache(self, mock_cache):
        mock_cache.get.return_value = None

        result = Notification.objects.user_unread_count(self.user_1.pk)
        expected = 2
        self.assertEqual(result, expected)
        self.assertTrue(mock_cache.set.called)
        args, kwargs = mock_cache.set.call_args
        self.assertEqual(
            kwargs['key'],
            Notification.objects._user_notification_cache_key(self.user_1.pk)
        )
        self.assertEqual(kwargs['value'], expected)

    def test_return_from_cache_when_in_cache(self, mock_cache):
        mock_cache.get.return_value = 42
        result = Notification.objects.user_unread_count(self.user_1.pk)
        expected = 42
        self.assertEqual(result, expected)
        self.assertFalse(mock_cache.set.called)

    def test_return_error_code_when_user_not_found(self, mock_cache):
        mock_cache.get.return_value = None
        invalid_user_id = max(user.pk for user in User.objects.all()) + 1
        result = Notification.objects.user_unread_count(invalid_user_id)
        expected = -1
        self.assertEqual(result, expected)
        self.assertFalse(mock_cache.set.called)

    def test_can_invalidate_cache(self, mock_cache):
        Notification.objects.invalidate_user_notification_cache(self.user_1.pk)
        self.assertTrue(mock_cache.delete)
        args, kwargs = mock_cache.delete.call_args
        self.assertEqual(
            kwargs['key'],
            Notification.objects._user_notification_cache_key(self.user_1.pk)
        )



# Generated by Django 2.2.10 on 2020-03-21 13:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('services', '0002_nameformatter'),
    ]

    operations = [
        migrations.AlterField(
            model_name='nameformatconfig',
            name='format',
            field=models.CharField(help_text='For information on constructing name formats please see the official documentation, topic "Services Name Formats".', max_length=100),
        ),
    ]



import os
from setuptools import setup
import allianceauth

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

install_requires = [
    'mysqlclient',
    'dnspython',
    'passlib',
    'requests>=2.9.1,<3.0.0',
    'bcrypt',
    'python-slugify>=1.2',
    'requests-oauthlib',
    'semantic_version',
    'packaging>=21.0,<22',
    'beautifulsoup4',

    'redis>=3.3.1,<4.0.0',
    'celery>=4.3.0,<6.0.0,!=4.4.4',  # 4.4.4 is missing a dependency
    'celery_once>=2.0.1',

    'django>=3.2.9,<4.0.0',
    'django-bootstrap-form',
    'django-registration>=3.1',
    'django-sortedm2m',
    'django-redis-cache>=3.0.0',
    'django-celery-beat>=2.0.0',

    'openfire-restapi',
    'sleekxmpp<=1.3.2',
    'pydiscourse',

    'django-esi>=4.0.1,<5.0.0'
]

testing_extras = [
    'coverage>=4.3.1',
    'requests-mock>=1.2.0',
    'django-webtest',
]

setup(
    name='allianceauth',
    version=allianceauth.__version__,
    author='Alliance Auth',
    author_email='adarnof@gmail.com',
    description=(
        'An auth system for EVE Online to help in-game organizations '
        'manage online service access.'
    ),
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=install_requires,
    extras_require={
        'testing': testing_extras
    },
    python_requires='~=3.7',
    license='GPLv2',
    packages=['allianceauth'],
    url=allianceauth.__url__,
    zip_safe=False,
    include_package_data=True,
    entry_points="""
            [console_scripts]
            allianceauth=allianceauth.bin.allianceauth:main
    """,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    project_urls={
        'Documentation': 'https://allianceauth.readthedocs.io/',
    },
)



from bs4 import BeautifulSoup

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from .models import AnalyticsTokens, AnalyticsIdentifier
from .tasks import send_ga_tracking_web_view

import re


class AnalyticsMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        """Django Middleware: Process Page Views and creates Analytics Celery Tasks"""
        if getattr(settings, "ANALYTICS_DISABLED", False):
            return response
        analyticstokens = AnalyticsTokens.objects.all()
        client_id = AnalyticsIdentifier.objects.get(id=1).identifier.hex
        try:
            title = BeautifulSoup(
                response.content, "html.parser").html.head.title.text
        except AttributeError:
            title = ''
        for token in analyticstokens:
            # Check if Page View Sending is Disabled
            if token.send_page_views is False:
                continue
            # Check Exclusions
            ignore = False
            for ignore_path in token.ignore_paths.values():
                ignore_path_regex = re.compile(ignore_path["ignore_path"])
                if re.search(ignore_path_regex, request.path) is not None:
                    ignore = True

            if ignore is True:
                continue

            tracking_id = token.token
            locale = request.LANGUAGE_CODE
            path = request.path
            try:
                useragent = request.headers["User-Agent"]
            except KeyError:
                useragent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36"

            send_ga_tracking_web_view.s(tracking_id=tracking_id,
                                        client_id=client_id,
                                        page=path,
                                        title=title,
                                        locale=locale,
                                        useragent=useragent).\
                apply_async(priority=9)
        return response



# Generated by Django 1.10.1 on 2016-09-10 20:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='eveapikeypair',
            name='error_count',
        ),
    ]



# Generated by Django 3.2.8 on 2021-10-19 01:47

from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations


def modify_aa_team_token_add_page_ignore_paths(apps, schema_editor):
    # Add the /account/activate path to ignore

    AnalyticsPath = apps.get_model('analytics', 'AnalyticsPath')
    account_activate = AnalyticsPath.objects.create(ignore_path=r"^\/account\/activate\/.*")

    Tokens = apps.get_model('analytics', 'AnalyticsTokens')
    token = Tokens.objects.get(token="UA-186249766-2")
    token.ignore_paths.add(account_activate)


def undo_modify_aa_team_token_add_page_ignore_paths(apps, schema_editor):
    #
    AnalyticsPath = apps.get_model('analytics', 'AnalyticsPath')
    Tokens = apps.get_model('analytics', 'AnalyticsTokens')

    token = Tokens.objects.get(token="UA-186249766-2")

    try:
        account_activate = AnalyticsPath.objects.get(ignore_path=r"^\/account\/activate\/.*", analyticstokens=token)
        account_activate.delete()
    except ObjectDoesNotExist:
        # Its fine if it doesnt exist, we just dont want them building up when re-migrating
        pass

class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0005_alter_analyticspath_ignore_path'),
    ]

    operations = [
        migrations.RunPython(modify_aa_team_token_add_page_ignore_paths, undo_modify_aa_team_token_add_page_ignore_paths)
    ]



from django.conf.urls import url, include

from . import views

app_name = 'teamspeak3'

module_urls = [
    # Teamspeak3 service control
    url(r'^activate/$', views.activate_teamspeak3, name='activate'),
    url(r'^deactivate/$', views.deactivate_teamspeak3, name='deactivate'),
    url(r'^reset_perm/$', views.reset_teamspeak3_perm, name='reset_perm'),
    url(
        r'^admin_update_ts3_groups/$',
        views.admin_update_ts3_groups,
        name='admin_update_ts3_groups'
    ),

    # Teamspeak Urls
    url(r'^verify/$', views.verify_teamspeak3, name='verify'),
]

urlpatterns = [
    url(r'^teamspeak3/', include((module_urls, app_name), namespace=app_name)),
]



# Generated by Django 3.1.12 on 2021-07-01 21:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0004_performance_tuning'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='level',
            field=models.CharField(choices=[('danger', 'danger'), ('warning', 'warning'), ('info', 'info'), ('success', 'success')], default='info', max_length=10),
        ),
    ]



import logging

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from .managers import NotificationManager

logger = logging.getLogger(__name__)


class Notification(models.Model):
    """Notification to a user within Auth"""

    NOTIFICATIONS_MAX_PER_USER_DEFAULT = 50
    NOTIFICATIONS_REFRESH_TIME_DEFAULT = 30

    class Level(models.TextChoices):
        """A notification level."""

        DANGER = 'danger', _('danger')  #:
        WARNING = 'warning', _('warning')  #:
        INFO = 'info', _('info')  #:
        SUCCESS = 'success', _('success')  #:

        @classmethod
        def from_old_name(cls, name: str) -> object:
            """Map old name to enum.

            Raises ValueError for invalid names.
            """
            name_map = {
                "CRITICAL": cls.DANGER,
                "ERROR": cls.DANGER,
                "WARN": cls.WARNING,
                "INFO": cls.INFO,
                "DEBUG": cls.SUCCESS,
            }
            try:
                return name_map[name]
            except KeyError:
                raise ValueError(f"Unknown name: {name}") from None

    # LEVEL_CHOICES = (
    #     ('danger', 'CRITICAL'),
    #     ('danger', 'ERROR'),
    #     ('warning', 'WARN'),
    #     ('info', 'INFO'),
    #     ('success', 'DEBUG'),
    # )

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    level = models.CharField(choices=Level.choices, max_length=10, default=Level.INFO)
    title = models.CharField(max_length=254)
    message = models.TextField()
    timestamp = models.DateTimeField(auto_now_add=True, db_index=True)
    viewed = models.BooleanField(default=False, db_index=True)

    objects = NotificationManager()

    def __str__(self) -> str:
        return f"{self.user}: {self.title}"

    def save(self, *args, **kwargs):
        # overriden save to ensure cache is invaidated on very call
        super().save(*args, **kwargs)
        Notification.objects.invalidate_user_notification_cache(self.user.pk)

    def delete(self, *args, **kwargs):
        # overriden delete to ensure cache is invaidated on very call
        super().delete(*args, **kwargs)
        Notification.objects.invalidate_user_notification_cache(self.user.pk)

    def mark_viewed(self) -> None:
        """Mark notification as viewed."""
        logger.info("Marking notification as viewed: %s" % self)
        self.viewed = True
        self.save()

    def set_level(self, level_name: str) -> None:
        """Set notification level according to old level name, e.g. 'CRITICAL'.

        Raises ValueError on invalid level names.
        """
        self.level = self.Level.from_old_name(level_name)
        self.save()



import datetime as dt

from pytz import utc
from django.test import TestCase
from django.utils.timezone import now

from allianceauth.authentication.task_statistics.event_series import EventSeries


class TestEventSeries(TestCase):
    def test_should_add_event(self):
        # given
        events = EventSeries("dummy")
        # when
        events.add()
        # then
        result = events.all()
        self.assertEqual(len(result), 1)
        self.assertAlmostEqual(result[0], now(), delta=dt.timedelta(seconds=30))

    def test_should_add_event_with_specified_time(self):
        # given
        events = EventSeries("dummy")
        my_time = dt.datetime(2021, 11, 1, 12, 15, tzinfo=utc)
        # when
        events.add(my_time)
        # then
        result = events.all()
        self.assertEqual(len(result), 1)
        self.assertAlmostEqual(result[0], my_time, delta=dt.timedelta(seconds=30))

    def test_should_count_events(self):
        # given
        events = EventSeries("dummy")
        events.add()
        events.add()
        # when
        result = events.count()
        # then
        self.assertEqual(result, 2)

    def test_should_count_zero(self):
        # given
        events = EventSeries("dummy")
        # when
        result = events.count()
        # then
        self.assertEqual(result, 0)

    def test_should_count_events_within_timeframe_1(self):
        # given
        events = EventSeries("dummy")
        events.add(dt.datetime(2021, 12, 1, 12, 0, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 10, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 15, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 30, tzinfo=utc))
        # when
        result = events.count(
            earliest=dt.datetime(2021, 12, 1, 12, 8, tzinfo=utc),
            latest=dt.datetime(2021, 12, 1, 12, 17, tzinfo=utc),
        )
        # then
        self.assertEqual(result, 2)

    def test_should_count_events_within_timeframe_2(self):
        # given
        events = EventSeries("dummy")
        events.add(dt.datetime(2021, 12, 1, 12, 0, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 10, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 15, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 30, tzinfo=utc))
        # when
        result = events.count(earliest=dt.datetime(2021, 12, 1, 12, 8))
        # then
        self.assertEqual(result, 3)

    def test_should_count_events_within_timeframe_3(self):
        # given
        events = EventSeries("dummy")
        events.add(dt.datetime(2021, 12, 1, 12, 0, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 10, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 15, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 30, tzinfo=utc))
        # when
        result = events.count(latest=dt.datetime(2021, 12, 1, 12, 12))
        # then
        self.assertEqual(result, 2)

    def test_should_clear_events(self):
        # given
        events = EventSeries("dummy")
        events.add()
        events.add()
        # when
        events.clear()
        # then
        self.assertEqual(events.count(), 0)

    def test_should_return_date_of_first_event(self):
        # given
        events = EventSeries("dummy")
        events.add(dt.datetime(2021, 12, 1, 12, 0, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 10, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 15, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 30, tzinfo=utc))
        # when
        result = events.first_event()
        # then
        self.assertEqual(result, dt.datetime(2021, 12, 1, 12, 0, tzinfo=utc))

    def test_should_return_date_of_first_event_with_range(self):
        # given
        events = EventSeries("dummy")
        events.add(dt.datetime(2021, 12, 1, 12, 0, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 10, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 15, tzinfo=utc))
        events.add(dt.datetime(2021, 12, 1, 12, 30, tzinfo=utc))
        # when
        result = events.first_event(
            earliest=dt.datetime(2021, 12, 1, 12, 8, tzinfo=utc)
        )
        # then
        self.assertEqual(result, dt.datetime(2021, 12, 1, 12, 10, tzinfo=utc))

    def test_should_return_all_events(self):
        # given
        events = EventSeries("dummy")
        events.add()
        events.add()
        # when
        results = events.all()
        # then
        self.assertEqual(len(results), 2)



from allianceauth.analytics.tasks import (
    analytics_event,
    send_ga_tracking_celery_event,
    send_ga_tracking_web_view)
from django.test.testcases import TestCase


class TestAnalyticsTasks(TestCase):
    def test_analytics_event(self):
        analytics_event(
                        category='allianceauth.analytics',
                        action='send_tests',
                        label='test',
                        value=1,
                        event_type='Stats')

    def test_send_ga_tracking_web_view_sent(self):
        # This test sends if the event SENDS to google
        # Not if it was successful
        tracking_id = 'UA-186249766-2'
        client_id = 'ab33e241fbf042b6aa77c7655a768af7'
        page = '/index/'
        title = 'Hello World'
        locale = 'en'
        useragent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36"
        response = send_ga_tracking_web_view(
                                            tracking_id,
                                            client_id,
                                            page,
                                            title,
                                            locale,
                                            useragent)
        self.assertEqual(response.status_code, 200)

    def test_send_ga_tracking_web_view_success(self):
        tracking_id = 'UA-186249766-2'
        client_id = 'ab33e241fbf042b6aa77c7655a768af7'
        page = '/index/'
        title = 'Hello World'
        locale = 'en'
        useragent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36"
        json_response = send_ga_tracking_web_view(
                                                tracking_id,
                                                client_id,
                                                page,
                                                title,
                                                locale,
                                                useragent).json()
        self.assertTrue(json_response["hitParsingResult"][0]["valid"])

    def test_send_ga_tracking_web_view_invalid_token(self):
        tracking_id = 'UA-IntentionallyBadTrackingID-2'
        client_id = 'ab33e241fbf042b6aa77c7655a768af7'
        page = '/index/'
        title = 'Hello World'
        locale = 'en'
        useragent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36"
        json_response = send_ga_tracking_web_view(
                                            tracking_id,
                                            client_id,
                                            page,
                                            title,
                                            locale,
                                            useragent).json()
        self.assertFalse(json_response["hitParsingResult"][0]["valid"])
        self.assertEqual(json_response["hitParsingResult"][0]["parserMessage"][1]["description"], "The value provided for parameter 'tid' is invalid. Please see http://goo.gl/a8d4RP#tid for details.")

        # [{'valid': False, 'parserMessage': [{'messageType': 'INFO', 'description': 'IP Address from this hit was anonymized to 1.132.110.0.', 'messageCode': 'VALUE_MODIFIED'}, {'messageType': 'ERROR', 'description': "The value provided for parameter 'tid' is invalid. Please see http://goo.gl/a8d4RP#tid for details.", 'messageCode': 'VALUE_INVALID', 'parameter': 'tid'}], 'hit': '/debug/collect?v=1&tid=UA-IntentionallyBadTrackingID-2&cid=ab33e241fbf042b6aa77c7655a768af7&t=pageview&dp=/index/&dt=Hello World&ul=en&ua=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36&aip=1&an=allianceauth&av=2.9.0a2'}]

    def test_send_ga_tracking_celery_event_sent(self):
        tracking_id = 'UA-186249766-2'
        client_id = 'ab33e241fbf042b6aa77c7655a768af7'
        category = 'test'
        action = 'test'
        label = 'test'
        value = '1'
        response = send_ga_tracking_celery_event(
                                                tracking_id,
                                                client_id,
                                                category,
                                                action,
                                                label,
                                                value)
        self.assertEqual(response.status_code, 200)

    def test_send_ga_tracking_celery_event_success(self):
        tracking_id = 'UA-186249766-2'
        client_id = 'ab33e241fbf042b6aa77c7655a768af7'
        category = 'test'
        action = 'test'
        label = 'test'
        value = '1'
        json_response = send_ga_tracking_celery_event(
                                                    tracking_id,
                                                    client_id,
                                                    category,
                                                    action,
                                                    label,
                                                    value).json()
        self.assertTrue(json_response["hitParsingResult"][0]["valid"])

    def test_send_ga_tracking_celery_event_invalid_token(self):
        tracking_id = 'UA-IntentionallyBadTrackingID-2'
        client_id = 'ab33e241fbf042b6aa77c7655a768af7'
        category = 'test'
        action = 'test'
        label = 'test'
        value = '1'
        json_response = send_ga_tracking_celery_event(
                                                    tracking_id,
                                                    client_id,
                                                    category,
                                                    action,
                                                    label,
                                                    value).json()
        self.assertFalse(json_response["hitParsingResult"][0]["valid"])
        self.assertEqual(json_response["hitParsingResult"][0]["parserMessage"][1]["description"], "The value provided for parameter 'tid' is invalid. Please see http://goo.gl/a8d4RP#tid for details.")

        # [{'valid': False, 'parserMessage': [{'messageType': 'INFO', 'description': 'IP Address from this hit was anonymized to 1.132.110.0.', 'messageCode': 'VALUE_MODIFIED'}, {'messageType': 'ERROR', 'description': "The value provided for parameter 'tid' is invalid. Please see http://goo.gl/a8d4RP#tid for details.", 'messageCode': 'VALUE_INVALID', 'parameter': 'tid'}], 'hit': '/debug/collect?v=1&tid=UA-IntentionallyBadTrackingID-2&cid=ab33e241fbf042b6aa77c7655a768af7&t=pageview&dp=/index/&dt=Hello World&ul=en&ua=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36&aip=1&an=allianceauth&av=2.9.0a2'}]



import logging

from django.db import transaction
from django.db.models.signals import m2m_changed
from django.db.models.signals import post_delete
from django.db.models.signals import post_save
from django.dispatch import receiver
from allianceauth.authentication.signals import state_changed
from .tasks import Teamspeak3Tasks
from .models import AuthTS, StateGroup

logger = logging.getLogger(__name__)


def trigger_all_ts_update():
    logger.debug("Triggering update_all_groups")
    Teamspeak3Tasks.update_all_groups()


@receiver(m2m_changed, sender=AuthTS.ts_group.through)
def m2m_changed_authts_group(sender, instance, action, *args, **kwargs):
    logger.debug(f"Received m2m_changed from {instance} ts_group with action {action}")
    if action == "post_add" or action == "post_remove":
        transaction.on_commit(trigger_all_ts_update)


@receiver(post_save, sender=AuthTS)
def post_save_authts(sender, instance, *args, **kwargs):
    logger.debug("Received post_save from %s" % instance)
    transaction.on_commit(trigger_all_ts_update)


@receiver(post_delete, sender=AuthTS)
def post_delete_authts(sender, instance, *args, **kwargs):
    logger.debug("Received post_delete signal from %s" % instance)
    transaction.on_commit(trigger_all_ts_update)


# it's literally the same logic so just recycle the receiver
post_save.connect(post_save_authts, sender=StateGroup)
post_delete.connect(post_delete_authts, sender=StateGroup)


@receiver(state_changed)
def check_groups_on_state_change(sender, user, state, **kwargs):
    def trigger_update():
        Teamspeak3Tasks.update_groups.delay(user.pk)
    logger.debug(f"Received state_changed signal from {user}")
    transaction.on_commit(trigger_update)



# Generated by Django 1.10.4 on 2017-01-23 10:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0004_auto_20161214_1024'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='hashfn',
            field=models.CharField(default='sha1', max_length=20),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='pwhash',
            field=models.CharField(max_length=80),
        ),
    ]
